// A Bison parser, made by GNU Bison 3.8.2.

// Skeleton implementation for Bison LALR(1) parsers in C++

// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// As a special exception, you may create a larger work that contains
// part or all of the Bison parser skeleton and distribute that work
// under terms of your choice, so long as that work isn't itself a
// parser generator using the skeleton or a modified version thereof
// as a parser skeleton.  Alternatively, if you modify or redistribute
// the parser skeleton itself, you may (at your option) remove this
// special exception, which will cause the skeleton and the resulting
// Bison output files to be licensed under the GNU General Public
// License without this special exception.

// This special exception was added by the Free Software Foundation in
// version 2.2 of Bison.

// DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
// especially those whose name start with YY_ or yy_.  They are
// private implementation details that can be changed or removed.





#include "parser.hpp"


// Unqualified %code blocks.
#line 46 "asm/parser.y"

	#include <algorithm>
	#include <ctype.h>
	#include <inttypes.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <string_view>

	#include "extern/utf8decoder.hpp"
	#include "helpers.hpp"

	#include "asm/charmap.hpp"
	#include "asm/fixpoint.hpp"
	#include "asm/format.hpp"
	#include "asm/fstack.hpp"
	#include "asm/main.hpp"
	#include "asm/opt.hpp"
	#include "asm/output.hpp"
	#include "asm/section.hpp"
	#include "asm/symbol.hpp"
	#include "asm/warning.hpp"

	using namespace std::literals;

	yy::parser::symbol_type yylex(); // Provided by lexer.cpp

	static uint32_t strToNum(std::vector<int32_t> const &s);
	static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName);
	static size_t strlenUTF8(std::string const &str, bool printErrors);
	static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len);
	static size_t charlenUTF8(std::string const &str);
	static std::string charsubUTF8(std::string const &str, uint32_t pos);
	static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName);
	static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep);
	static std::string strfmt(
	    std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args
	);
	static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue);
	static void failAssert(AssertionType type);
	static void failAssertMsg(AssertionType type, std::string const &message);

	// The CPU encodes instructions in a logical way, so most instructions actually follow patterns.
	// These enums thus help with bit twiddling to compute opcodes.
	enum { REG_B, REG_C, REG_D, REG_E, REG_H, REG_L, REG_HL_IND, REG_A };

	enum { REG_BC_IND, REG_DE_IND, REG_HL_INDINC, REG_HL_INDDEC };

	// REG_AF == REG_SP since LD/INC/ADD/DEC allow SP, while PUSH/POP allow AF
	enum { REG_BC, REG_DE, REG_HL, REG_SP, REG_AF = REG_SP };

	// CC_NZ == CC_Z ^ 1, and CC_NC == CC_C ^ 1, so `!` can toggle them
	enum { CC_NZ, CC_Z, CC_NC, CC_C };

#line 101 "/build/rgbds/src/rgbds/src/asm/parser.cpp"


#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> // FIXME: INFRINGES ON USER NAME SPACE.
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif


// Whether we are compiled with exception support.
#ifndef YY_EXCEPTIONS
# if defined __GNUC__ && !defined __EXCEPTIONS
#  define YY_EXCEPTIONS 0
# else
#  define YY_EXCEPTIONS 1
# endif
#endif



// Enable debugging if requested.
#if YYDEBUG

// A pseudo ostream that takes yydebug_ into account.
# define YYCDEBUG if (yydebug_) (*yycdebug_)

# define YY_SYMBOL_PRINT(Title, Symbol)         \
  do {                                          \
    if (yydebug_)                               \
    {                                           \
      *yycdebug_ << Title << ' ';               \
      yy_print_ (*yycdebug_, Symbol);           \
      *yycdebug_ << '\n';                       \
    }                                           \
  } while (false)

# define YY_REDUCE_PRINT(Rule)          \
  do {                                  \
    if (yydebug_)                       \
      yy_reduce_print_ (Rule);          \
  } while (false)

# define YY_STACK_PRINT()               \
  do {                                  \
    if (yydebug_)                       \
      yy_stack_print_ ();                \
  } while (false)

#else // !YYDEBUG

# define YYCDEBUG if (false) std::cerr
# define YY_SYMBOL_PRINT(Title, Symbol)  YY_USE (Symbol)
# define YY_REDUCE_PRINT(Rule)           static_cast<void> (0)
# define YY_STACK_PRINT()                static_cast<void> (0)

#endif // !YYDEBUG

#define yyerrok         (yyerrstatus_ = 0)
#define yyclearin       (yyla.clear ())

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYRECOVERING()  (!!yyerrstatus_)

namespace yy {
#line 174 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

  /// Build a parser object.
  parser::parser ()
#if YYDEBUG
    : yydebug_ (false),
      yycdebug_ (&std::cerr),
#else
    :
#endif
      yy_lac_established_ (false)
  {}

  parser::~parser ()
  {}

  parser::syntax_error::~syntax_error () YY_NOEXCEPT YY_NOTHROW
  {}

  /*---------.
  | symbol.  |
  `---------*/



  // by_state.
  parser::by_state::by_state () YY_NOEXCEPT
    : state (empty_state)
  {}

  parser::by_state::by_state (const by_state& that) YY_NOEXCEPT
    : state (that.state)
  {}

  void
  parser::by_state::clear () YY_NOEXCEPT
  {
    state = empty_state;
  }

  void
  parser::by_state::move (by_state& that)
  {
    state = that.state;
    that.clear ();
  }

  parser::by_state::by_state (state_type s) YY_NOEXCEPT
    : state (s)
  {}

  parser::symbol_kind_type
  parser::by_state::kind () const YY_NOEXCEPT
  {
    if (state == empty_state)
      return symbol_kind::S_YYEMPTY;
    else
      return YY_CAST (symbol_kind_type, yystos_[+state]);
  }

  parser::stack_symbol_type::stack_symbol_type ()
  {}

  parser::stack_symbol_type::stack_symbol_type (YY_RVREF (stack_symbol_type) that)
    : super_type (YY_MOVE (that.state))
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.YY_MOVE_OR_COPY< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.YY_MOVE_OR_COPY< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.YY_MOVE_OR_COPY< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.YY_MOVE_OR_COPY< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.YY_MOVE_OR_COPY< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.YY_MOVE_OR_COPY< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.YY_MOVE_OR_COPY< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.YY_MOVE_OR_COPY< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.YY_MOVE_OR_COPY< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.YY_MOVE_OR_COPY< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.YY_MOVE_OR_COPY< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.YY_MOVE_OR_COPY< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.YY_MOVE_OR_COPY< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.YY_MOVE_OR_COPY< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.YY_MOVE_OR_COPY< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.YY_MOVE_OR_COPY< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

#if 201103L <= YY_CPLUSPLUS
    // that is emptied.
    that.state = empty_state;
#endif
  }

  parser::stack_symbol_type::stack_symbol_type (state_type s, YY_MOVE_REF (symbol_type) that)
    : super_type (s)
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

    // that is emptied.
    that.kind_ = symbol_kind::S_YYEMPTY;
  }

#if YY_CPLUSPLUS < 201103L
  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (const stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.copy< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.copy< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.copy< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.copy< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.copy< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.copy< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.copy< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.copy< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.copy< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.copy< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.copy< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.copy< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.copy< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.copy< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.copy< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.copy< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    return *this;
  }

  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    // that is emptied.
    that.state = empty_state;
    return *this;
  }
#endif

  template <typename Base>
  void
  parser::yy_destroy_ (const char* yymsg, basic_symbol<Base>& yysym) const
  {
    if (yymsg)
      YY_SYMBOL_PRINT (yymsg, yysym);
  }

#if YYDEBUG
  template <typename Base>
  void
  parser::yy_print_ (std::ostream& yyo, const basic_symbol<Base>& yysym) const
  {
    std::ostream& yyoutput = yyo;
    YY_USE (yyoutput);
    if (yysym.empty ())
      yyo << "empty symbol";
    else
      {
        symbol_kind_type yykind = yysym.kind ();
        yyo << (yykind < YYNTOKENS ? "token" : "nterm")
            << ' ' << yysym.name () << " (";
        YY_USE (yykind);
        yyo << ')';
      }
  }
#endif

  void
  parser::yypush_ (const char* m, YY_MOVE_REF (stack_symbol_type) sym)
  {
    if (m)
      YY_SYMBOL_PRINT (m, sym);
    yystack_.push (YY_MOVE (sym));
  }

  void
  parser::yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym)
  {
#if 201103L <= YY_CPLUSPLUS
    yypush_ (m, stack_symbol_type (s, std::move (sym)));
#else
    stack_symbol_type ss (s, sym);
    yypush_ (m, ss);
#endif
  }

  void
  parser::yypop_ (int n) YY_NOEXCEPT
  {
    yystack_.pop (n);
  }

#if YYDEBUG
  std::ostream&
  parser::debug_stream () const
  {
    return *yycdebug_;
  }

  void
  parser::set_debug_stream (std::ostream& o)
  {
    yycdebug_ = &o;
  }


  parser::debug_level_type
  parser::debug_level () const
  {
    return yydebug_;
  }

  void
  parser::set_debug_level (debug_level_type l)
  {
    yydebug_ = l;
  }
#endif // YYDEBUG

  parser::state_type
  parser::yy_lr_goto_state_ (state_type yystate, int yysym)
  {
    int yyr = yypgoto_[yysym - YYNTOKENS] + yystate;
    if (0 <= yyr && yyr <= yylast_ && yycheck_[yyr] == yystate)
      return yytable_[yyr];
    else
      return yydefgoto_[yysym - YYNTOKENS];
  }

  bool
  parser::yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yypact_ninf_;
  }

  bool
  parser::yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yytable_ninf_;
  }

  int
  parser::operator() ()
  {
    return parse ();
  }

  int
  parser::parse ()
  {
    int yyn;
    /// Length of the RHS of the rule being reduced.
    int yylen = 0;

    // Error handling.
    int yynerrs_ = 0;
    int yyerrstatus_ = 0;

    /// The lookahead symbol.
    symbol_type yyla;

    /// The return value of parse ().
    int yyresult;

    // Discard the LAC context in case there still is one left from a
    // previous invocation.
    yy_lac_discard_ ("init");

#if YY_EXCEPTIONS
    try
#endif // YY_EXCEPTIONS
      {
    YYCDEBUG << "Starting parse\n";


    /* Initialize the stack.  The initial state will be set in
       yynewstate, since the latter expects the semantical and the
       location values to have been already stored, initialize these
       stacks with a primary value.  */
    yystack_.clear ();
    yypush_ (YY_NULLPTR, 0, YY_MOVE (yyla));

  /*-----------------------------------------------.
  | yynewstate -- push a new symbol on the stack.  |
  `-----------------------------------------------*/
  yynewstate:
    YYCDEBUG << "Entering state " << int (yystack_[0].state) << '\n';
    YY_STACK_PRINT ();

    // Accept?
    if (yystack_[0].state == yyfinal_)
      YYACCEPT;

    goto yybackup;


  /*-----------.
  | yybackup.  |
  `-----------*/
  yybackup:
    // Try to take a decision without lookahead.
    yyn = yypact_[+yystack_[0].state];
    if (yy_pact_value_is_default_ (yyn))
      goto yydefault;

    // Read a lookahead token.
    if (yyla.empty ())
      {
        YYCDEBUG << "Reading a token\n";
#if YY_EXCEPTIONS
        try
#endif // YY_EXCEPTIONS
          {
            symbol_type yylookahead (yylex ());
            yyla.move (yylookahead);
          }
#if YY_EXCEPTIONS
        catch (const syntax_error& yyexc)
          {
            YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
            error (yyexc);
            goto yyerrlab1;
          }
#endif // YY_EXCEPTIONS
      }
    YY_SYMBOL_PRINT ("Next token is", yyla);

    if (yyla.kind () == symbol_kind::S_YYerror)
    {
      // The scanner already issued an error message, process directly
      // to error recovery.  But do not keep the error token as
      // lookahead, it is too special and may lead us to an endless
      // loop in error recovery. */
      yyla.kind_ = symbol_kind::S_YYUNDEF;
      goto yyerrlab1;
    }

    /* If the proper action on seeing token YYLA.TYPE is to reduce or
       to detect an error, take that action.  */
    yyn += yyla.kind ();
    if (yyn < 0 || yylast_ < yyn || yycheck_[yyn] != yyla.kind ())
      {
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;
        goto yydefault;
      }

    // Reduce or error.
    yyn = yytable_[yyn];
    if (yyn <= 0)
      {
        if (yy_table_value_is_error_ (yyn))
          goto yyerrlab;
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;

        yyn = -yyn;
        goto yyreduce;
      }

    // Count tokens shifted since error; after three, turn off error status.
    if (yyerrstatus_)
      --yyerrstatus_;

    // Shift the lookahead token.
    yypush_ ("Shifting", state_type (yyn), YY_MOVE (yyla));
    yy_lac_discard_ ("shift");
    goto yynewstate;


  /*-----------------------------------------------------------.
  | yydefault -- do the default action for the current state.  |
  `-----------------------------------------------------------*/
  yydefault:
    yyn = yydefact_[+yystack_[0].state];
    if (yyn == 0)
      goto yyerrlab;
    goto yyreduce;


  /*-----------------------------.
  | yyreduce -- do a reduction.  |
  `-----------------------------*/
  yyreduce:
    yylen = yyr2_[yyn];
    {
      stack_symbol_type yylhs;
      yylhs.state = yy_lr_goto_state_ (yystack_[yylen].state, yyr1_[yyn]);
      /* Variants are always initialized to an empty instance of the
         correct type. The default '$$ = $1' action is NOT applied
         when using variants.  */
      switch (yyr1_[yyn])
    {
      case symbol_kind::S_align_spec: // align_spec
        yylhs.value.emplace< AlignmentSpec > ();
        break;

      case symbol_kind::S_assert_type: // assert_type
        yylhs.value.emplace< AssertionType > ();
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        yylhs.value.emplace< Capture > ();
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        yylhs.value.emplace< Expression > ();
        break;

      case symbol_kind::S_for_args: // for_args
        yylhs.value.emplace< ForArgs > ();
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        yylhs.value.emplace< RPNCommand > ();
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        yylhs.value.emplace< SectionModifier > ();
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        yylhs.value.emplace< SectionSpec > ();
        break;

      case symbol_kind::S_sect_type: // sect_type
        yylhs.value.emplace< SectionType > ();
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        yylhs.value.emplace< StrFmtArgList > ();
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        yylhs.value.emplace< int32_t > ();
        break;

      case symbol_kind::S_macro_args: // macro_args
        yylhs.value.emplace< std::shared_ptr<MacroArgs> > ();
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        yylhs.value.emplace< std::string > ();
        break;

      case symbol_kind::S_ds_args: // ds_args
        yylhs.value.emplace< std::vector<Expression> > ();
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        yylhs.value.emplace< std::vector<int32_t> > ();
        break;

      case symbol_kind::S_purge_args: // purge_args
        yylhs.value.emplace< std::vector<std::string> > ();
        break;

      default:
        break;
    }



      // Perform the reduction.
      YY_REDUCE_PRINT (yyn);
#if YY_EXCEPTIONS
      try
#endif // YY_EXCEPTIONS
        {
          switch (yyn)
            {
  case 5: // $@1: %empty
#line 406 "asm/parser.y"
                {
		lexer_SetMode(LEXER_NORMAL);
		lexer_ToggleStringExpansion(true);
	}
#line 1119 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 6: // lines: error $@1 endofline
#line 409 "asm/parser.y"
                    {
		fstk_StopRept();
		yyerrok;
	}
#line 1128 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 8: // diff_mark: "+"
#line 417 "asm/parser.y"
                 {
		::error(
			"syntax error, unexpected + at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1138 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 9: // diff_mark: "-"
#line 422 "asm/parser.y"
                 {
		::error(
			"syntax error, unexpected - at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1148 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 22: // if: "IF" iconst "end of line"
#line 455 "asm/parser.y"
                              {
		lexer_IncIFDepth();

		if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1162 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 23: // elif: "ELIF" iconst "end of line"
#line 467 "asm/parser.y"
                                {
		if (lexer_GetIFDepth() == 0) {
			fatalerror("Found ELIF outside of an IF construct\n");
		}
		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock()) {
				fatalerror("Found ELIF after an ELSE block\n");
			}
			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1182 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 24: // else: "ELSE" "end of line"
#line 485 "asm/parser.y"
                         {
		if (lexer_GetIFDepth() == 0) {
			fatalerror("Found ELSE outside of an IF construct\n");
		}
		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock()) {
				fatalerror("Found ELSE after an ELSE block\n");
			}
			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else {
			lexer_RunIFBlock();
			lexer_ReachELSEBlock();
		}
	}
#line 1201 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 29: // endc: "ENDC"
#line 511 "asm/parser.y"
                 {
		lexer_DecIFDepth();
	}
#line 1209 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 30: // $@2: %empty
#line 517 "asm/parser.y"
               {
		lexer_ToggleStringExpansion(false);
	}
#line 1217 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 31: // def_id: "DEF" $@2 "identifier"
#line 519 "asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1226 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 32: // $@3: %empty
#line 526 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1234 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 33: // redef_id: "REDEF" $@3 "identifier"
#line 528 "asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1243 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 34: // scoped_id: "identifier"
#line 537 "asm/parser.y"
           {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1251 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 35: // scoped_id: "local identifier"
#line 540 "asm/parser.y"
                   {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1259 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 36: // scoped_id: "label"
#line 543 "asm/parser.y"
                {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1267 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 37: // scoped_anon_id: scoped_id
#line 549 "asm/parser.y"
                  {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1275 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 38: // scoped_anon_id: "anonymous label"
#line 552 "asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1283 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 40: // label: ":"
#line 559 "asm/parser.y"
                {
		sym_AddAnonLabel();
	}
#line 1291 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 41: // label: "local identifier"
#line 562 "asm/parser.y"
                   {
		sym_AddLocalLabel(yystack_[0].value.as < std::string > ());
	}
#line 1299 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 42: // label: "local identifier" ":"
#line 565 "asm/parser.y"
                         {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
	}
#line 1307 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 43: // label: "label" ":"
#line 568 "asm/parser.y"
                      {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
	}
#line 1315 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 44: // label: "local identifier" "::"
#line 571 "asm/parser.y"
                                {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1324 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 45: // label: "label" "::"
#line 575 "asm/parser.y"
                             {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1333 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 46: // $@4: %empty
#line 582 "asm/parser.y"
           {
		// Parsing 'macro_args' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1342 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 47: // macro: "identifier" $@4 macro_args
#line 585 "asm/parser.y"
                     {
		fstk_RunMacro(yystack_[2].value.as < std::string > (), yystack_[0].value.as < std::shared_ptr<MacroArgs> > ());
	}
#line 1350 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 48: // macro_args: %empty
#line 591 "asm/parser.y"
               {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::make_shared<MacroArgs>();
	}
#line 1358 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 49: // macro_args: macro_args "string"
#line 594 "asm/parser.y"
                            {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::move(yystack_[1].value.as < std::shared_ptr<MacroArgs> > ());
		yylhs.value.as < std::shared_ptr<MacroArgs> > ()->appendArg(std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1367 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 89: // def_numeric: def_equ
#line 643 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1373 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 90: // def_numeric: redef_equ
#line 644 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1379 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 91: // def_numeric: def_set
#line 645 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1385 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 92: // def_numeric: def_rb
#line 646 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1391 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 93: // def_numeric: def_rw
#line 647 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1397 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 94: // def_numeric: def_rl
#line 648 "asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1403 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 97: // compound_eq: "+="
#line 654 "asm/parser.y"
                  {
		yylhs.value.as < RPNCommand > () = RPN_ADD;
	}
#line 1411 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 98: // compound_eq: "-="
#line 657 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SUB;
	}
#line 1419 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 99: // compound_eq: "*="
#line 660 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MUL;
	}
#line 1427 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 100: // compound_eq: "/="
#line 663 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_DIV;
	}
#line 1435 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 101: // compound_eq: "%="
#line 666 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MOD;
	}
#line 1443 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 102: // compound_eq: "^="
#line 669 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_XOR;
	}
#line 1451 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 103: // compound_eq: "|="
#line 672 "asm/parser.y"
                   {
		yylhs.value.as < RPNCommand > () = RPN_OR;
	}
#line 1459 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 104: // compound_eq: "&="
#line 675 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_AND;
	}
#line 1467 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 105: // compound_eq: "<<="
#line 678 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHL;
	}
#line 1475 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 106: // compound_eq: ">>="
#line 681 "asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHR;
	}
#line 1483 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 107: // align: "ALIGN" align_spec
#line 687 "asm/parser.y"
                             {
		sect_AlignPC(yystack_[0].value.as < AlignmentSpec > ().alignment, yystack_[0].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1491 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 108: // align_spec: uconst
#line 693 "asm/parser.y"
               {
		if (yystack_[0].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[0].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[0].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		}
	}
#line 1505 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 109: // align_spec: uconst "," iconst
#line 702 "asm/parser.y"
                              {
		if (yystack_[2].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[2].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else if (yystack_[0].value.as < int32_t > () <= -(1 << yystack_[2].value.as < int32_t > ()) || yystack_[0].value.as < int32_t > () >= 1 << yystack_[2].value.as < int32_t > ()) {
			::error(
				"The absolute alignment offset (%" PRIu32 ") must be less than alignment size (%d)\n",
				static_cast<uint32_t>(yystack_[0].value.as < int32_t > () < 0 ? -yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ()),
				1 << yystack_[2].value.as < int32_t > ()
			);
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[2].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = yystack_[0].value.as < int32_t > () < 0 ? (1 << yystack_[2].value.as < int32_t > ()) + yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ();
		}
	}
#line 1526 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 110: // $@5: %empty
#line 721 "asm/parser.y"
                {
		// Parsing 'opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1535 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 114: // opt_list_entry: "string"
#line 733 "asm/parser.y"
               {
		opt_Parse(yystack_[0].value.as < std::string > ().c_str());
	}
#line 1543 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 115: // popo: "POPO"
#line 739 "asm/parser.y"
                 {
		opt_Pop();
	}
#line 1551 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 116: // $@6: %empty
#line 745 "asm/parser.y"
                  {
		opt_Push();
		// Parsing 'pusho_opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1561 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 118: // pusho_opt_list: %empty
#line 753 "asm/parser.y"
               {
		lexer_SetMode(LEXER_NORMAL);
	}
#line 1569 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 120: // pops: "POPS"
#line 760 "asm/parser.y"
                 {
		sect_PopSection();
	}
#line 1577 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 121: // pushs: "PUSHS"
#line 766 "asm/parser.y"
                  {
		sect_PushSection();
	}
#line 1585 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 122: // endsection: "ENDSECTION"
#line 772 "asm/parser.y"
                       {
		sect_EndSection();
	}
#line 1593 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 123: // fail: "FAIL" string
#line 778 "asm/parser.y"
                        {
		fatalerror("%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1601 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 124: // warn: "WARN" string
#line 784 "asm/parser.y"
                        {
		warning(WARNING_USER, "%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1609 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 125: // assert_type: %empty
#line 790 "asm/parser.y"
               {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1617 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 126: // assert_type: "WARN" ","
#line 793 "asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_WARN;
	}
#line 1625 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 127: // assert_type: "FAIL" ","
#line 796 "asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1633 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 128: // assert_type: "FATAL" ","
#line 799 "asm/parser.y"
                          {
		yylhs.value.as < AssertionType > () = ASSERT_FATAL;
	}
#line 1641 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 129: // assert: "ASSERT" assert_type relocexpr
#line 805 "asm/parser.y"
                                         {
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[1].value.as < AssertionType > (), yystack_[0].value.as < Expression > (), "", sect_GetOutputOffset());
		} else if (yystack_[0].value.as < Expression > ().value() == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1653 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 130: // assert: "ASSERT" assert_type relocexpr "," string
#line 812 "asm/parser.y"
                                                        {
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[3].value.as < AssertionType > (), yystack_[2].value.as < Expression > (), yystack_[0].value.as < std::string > (), sect_GetOutputOffset());
		} else if (yystack_[2].value.as < Expression > ().value() == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1665 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 131: // assert: "STATIC_ASSERT" assert_type iconst
#line 819 "asm/parser.y"
                                               {
		if (yystack_[0].value.as < int32_t > () == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1675 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 132: // assert: "STATIC_ASSERT" assert_type iconst "," string
#line 824 "asm/parser.y"
                                                            {
		if (yystack_[2].value.as < int32_t > () == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1685 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 133: // shift: "SHIFT" shift_const
#line 832 "asm/parser.y"
                              {
		if (MacroArgs *macroArgs = fstk_GetCurrentMacroArgs(); macroArgs) {
			macroArgs->shiftArgs(yystack_[0].value.as < int32_t > ());
		} else {
			::error("Cannot shift macro arguments outside of a macro\n");
		}
	}
#line 1697 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 134: // shift_const: %empty
#line 842 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1705 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 135: // shift_const: iconst
#line 845 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1711 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 136: // load: "LOAD" sect_mod string "," sect_type sect_org sect_attrs
#line 849 "asm/parser.y"
                                                                     {
		sect_SetLoadSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 1719 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 137: // load: "ENDL"
#line 852 "asm/parser.y"
                   {
		sect_EndLoadSection(nullptr);
	}
#line 1727 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 138: // rept: "REPT" uconst "end of line" capture_rept endofline
#line 858 "asm/parser.y"
                                                       {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			fstk_RunRept(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1737 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 139: // $@7: %empty
#line 866 "asm/parser.y"
                {
		lexer_ToggleStringExpansion(false);
	}
#line 1745 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 140: // $@8: %empty
#line 868 "asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1753 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 141: // for: "FOR" $@7 "identifier" $@8 "," for_args "end of line" capture_rept endofline
#line 870 "asm/parser.y"
                                                        {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			fstk_RunFor(yystack_[6].value.as < std::string > (), yystack_[3].value.as < ForArgs > ().start, yystack_[3].value.as < ForArgs > ().stop, yystack_[3].value.as < ForArgs > ().step, yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1763 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 142: // capture_rept: %empty
#line 878 "asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureRept();
	}
#line 1771 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 143: // for_args: iconst
#line 884 "asm/parser.y"
               {
		yylhs.value.as < ForArgs > ().start = 0;
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1781 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 144: // for_args: iconst "," iconst
#line 889 "asm/parser.y"
                              {
		yylhs.value.as < ForArgs > ().start = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1791 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 145: // for_args: iconst "," iconst "," iconst
#line 894 "asm/parser.y"
                                           {
		yylhs.value.as < ForArgs > ().start = yystack_[4].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = yystack_[0].value.as < int32_t > ();
	}
#line 1801 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 146: // break: label "BREAK" endofline
#line 902 "asm/parser.y"
                                  {
		if (fstk_Break()) {
			lexer_SetMode(LEXER_SKIP_TO_ENDR);
		}
	}
#line 1811 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 147: // $@9: %empty
#line 910 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1819 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 148: // $@10: %empty
#line 912 "asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1827 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 149: // def_macro: "MACRO" $@9 "identifier" $@10 "end of line" capture_macro endofline
#line 914 "asm/parser.y"
                                          {
		if (yystack_[1].value.as < Capture > ().span.ptr) {
			sym_AddMacro(yystack_[4].value.as < std::string > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
		}
	}
#line 1837 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 150: // capture_macro: %empty
#line 922 "asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureMacro();
	}
#line 1845 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 151: // rsset: "RSSET" uconst
#line 928 "asm/parser.y"
                         {
		sym_SetRSValue(yystack_[0].value.as < int32_t > ());
	}
#line 1853 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 152: // rsreset: "RSRESET"
#line 934 "asm/parser.y"
                    {
		sym_SetRSValue(0);
	}
#line 1861 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 153: // rs_uconst: %empty
#line 940 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1869 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 154: // rs_uconst: uconst
#line 943 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1875 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 155: // union: "UNION"
#line 947 "asm/parser.y"
                  {
		sect_StartUnion();
	}
#line 1883 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 156: // nextu: "NEXTU"
#line 953 "asm/parser.y"
                  {
		sect_NextUnionMember();
	}
#line 1891 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 157: // endu: "ENDU"
#line 959 "asm/parser.y"
                 {
		sect_EndUnion();
	}
#line 1899 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 158: // ds: "DS" uconst
#line 965 "asm/parser.y"
                      {
		sect_Skip(yystack_[0].value.as < int32_t > (), true);
	}
#line 1907 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 159: // ds: "DS" uconst "," ds_args trailing_comma
#line 968 "asm/parser.y"
                                                     {
		sect_RelBytes(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < std::vector<Expression> > ());
	}
#line 1915 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 160: // ds: "DS" "ALIGN" "[" align_spec "]" trailing_comma
#line 971 "asm/parser.y"
                                                                   {
		uint32_t n = sect_GetAlignBytes(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);

		sect_Skip(n, true);
		sect_AlignPC(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1926 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 161: // ds: "DS" "ALIGN" "[" align_spec "]" "," ds_args trailing_comma
#line 977 "asm/parser.y"
                                                                                 {
		uint32_t n = sect_GetAlignBytes(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);

		sect_RelBytes(n, yystack_[1].value.as < std::vector<Expression> > ());
		sect_AlignPC(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1937 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 162: // ds_args: reloc_8bit
#line 986 "asm/parser.y"
                   {
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1945 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 163: // ds_args: ds_args "," reloc_8bit
#line 989 "asm/parser.y"
                                   {
		yylhs.value.as < std::vector<Expression> > () = std::move(yystack_[2].value.as < std::vector<Expression> > ());
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1954 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 164: // db: "DB"
#line 996 "asm/parser.y"
               {
		sect_Skip(1, false);
	}
#line 1962 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 166: // dw: "DW"
#line 1003 "asm/parser.y"
               {
		sect_Skip(2, false);
	}
#line 1970 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 168: // dl: "DL"
#line 1010 "asm/parser.y"
               {
		sect_Skip(4, false);
	}
#line 1978 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 170: // def_equ: def_id "EQU" iconst
#line 1017 "asm/parser.y"
                              {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1987 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 171: // redef_equ: redef_id "EQU" iconst
#line 1024 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1996 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 172: // def_set: def_id "=" iconst
#line 1031 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2005 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 173: // def_set: redef_id "=" iconst
#line 1035 "asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2014 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 174: // def_set: def_id compound_eq iconst
#line 1039 "asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2023 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 175: // def_set: redef_id compound_eq iconst
#line 1043 "asm/parser.y"
                                      {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2032 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 176: // def_rb: def_id "RB" rs_uconst
#line 1050 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + yystack_[0].value.as < int32_t > ());
	}
#line 2043 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 177: // def_rw: def_id "RW" rs_uconst
#line 1059 "asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 2 * yystack_[0].value.as < int32_t > ());
	}
#line 2054 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 178: // def_rl: def_id "rl" rs_uconst
#line 1068 "asm/parser.y"
                                 {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 4 * yystack_[0].value.as < int32_t > ());
	}
#line 2065 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 179: // def_equs: def_id "EQUS" string
#line 1077 "asm/parser.y"
                               {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2074 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 180: // redef_equs: redef_id "EQUS" string
#line 1084 "asm/parser.y"
                                 {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2083 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 181: // $@11: %empty
#line 1091 "asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 2091 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 182: // purge: "PURGE" $@11 purge_args trailing_comma
#line 1093 "asm/parser.y"
                                    {
		for (std::string &arg : yystack_[1].value.as < std::vector<std::string> > ()) {
			sym_Purge(arg);
		}
		lexer_ToggleStringExpansion(true);
	}
#line 2102 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 183: // purge_args: scoped_id
#line 1102 "asm/parser.y"
                  {
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2110 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 184: // purge_args: purge_args "," scoped_id
#line 1105 "asm/parser.y"
                                     {
		yylhs.value.as < std::vector<std::string> > () = std::move(yystack_[2].value.as < std::vector<std::string> > ());
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2119 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 188: // export_list_entry: scoped_id
#line 1119 "asm/parser.y"
                  {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2127 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 189: // export_def: "EXPORT" def_numeric
#line 1125 "asm/parser.y"
                               {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2135 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 190: // include: label "INCLUDE" string endofline
#line 1131 "asm/parser.y"
                                           {
		fstk_RunInclude(yystack_[1].value.as < std::string > (), false);
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2146 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 191: // incbin: "INCBIN" string
#line 1140 "asm/parser.y"
                          {
		sect_BinaryFile(yystack_[0].value.as < std::string > (), 0);
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2157 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 192: // incbin: "INCBIN" string "," iconst
#line 1146 "asm/parser.y"
                                         {
		sect_BinaryFile(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2168 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 193: // incbin: "INCBIN" string "," iconst "," iconst
#line 1152 "asm/parser.y"
                                                      {
		sect_BinaryFileSlice(yystack_[4].value.as < std::string > (), yystack_[2].value.as < int32_t > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude) {
			YYACCEPT;
		}
	}
#line 2179 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 194: // charmap: "CHARMAP" string "," charmap_args trailing_comma
#line 1161 "asm/parser.y"
                                                             {
		charmap_Add(yystack_[3].value.as < std::string > (), std::move(yystack_[1].value.as < std::vector<int32_t> > ()));
	}
#line 2187 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 195: // charmap_args: iconst
#line 1167 "asm/parser.y"
               {
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2195 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 196: // charmap_args: charmap_args "," iconst
#line 1170 "asm/parser.y"
                                    {
		yylhs.value.as < std::vector<int32_t> > () = std::move(yystack_[2].value.as < std::vector<int32_t> > ());
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2204 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 197: // newcharmap: "NEWCHARMAP" "identifier"
#line 1177 "asm/parser.y"
                          {
		charmap_New(yystack_[0].value.as < std::string > (), nullptr);
	}
#line 2212 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 198: // newcharmap: "NEWCHARMAP" "identifier" "," "identifier"
#line 1180 "asm/parser.y"
                                     {
		charmap_New(yystack_[2].value.as < std::string > (), &yystack_[0].value.as < std::string > ());
	}
#line 2220 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 199: // setcharmap: "SETCHARMAP" "identifier"
#line 1186 "asm/parser.y"
                          {
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2228 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 200: // pushc: "PUSHC"
#line 1192 "asm/parser.y"
                  {
		charmap_Push();
	}
#line 2236 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 201: // pushc_setcharmap: "PUSHC" "identifier"
#line 1198 "asm/parser.y"
                     {
		charmap_Push();
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2245 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 202: // popc: "POPC"
#line 1205 "asm/parser.y"
                 {
		charmap_Pop();
	}
#line 2253 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 204: // println: "PRINTLN"
#line 1213 "asm/parser.y"
                    {
		putchar('\n');
		fflush(stdout);
	}
#line 2262 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 205: // println: "PRINTLN" print_exprs trailing_comma
#line 1217 "asm/parser.y"
                                                 {
		putchar('\n');
		fflush(stdout);
	}
#line 2271 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 208: // print_expr: const_no_str
#line 1229 "asm/parser.y"
                     {
		printf("$%" PRIX32, yystack_[0].value.as < int32_t > ());
	}
#line 2279 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 209: // print_expr: string
#line 1232 "asm/parser.y"
                 {
		// Allow printing NUL characters
		fwrite(yystack_[0].value.as < std::string > ().data(), 1, yystack_[0].value.as < std::string > ().length(), stdout);
	}
#line 2288 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 210: // bit_const: iconst
#line 1239 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 7) {
			::error("Bit number must be between 0 and 7, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = 0;
		}
	}
#line 2300 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 213: // constlist_8bit_entry: reloc_8bit_no_str
#line 1254 "asm/parser.y"
                          {
		sect_RelByte(yystack_[0].value.as < Expression > (), 0);
	}
#line 2308 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 214: // constlist_8bit_entry: string
#line 1257 "asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_ByteString(output);
	}
#line 2317 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 217: // constlist_16bit_entry: reloc_16bit_no_str
#line 1269 "asm/parser.y"
                           {
		sect_RelWord(yystack_[0].value.as < Expression > (), 0);
	}
#line 2325 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 218: // constlist_16bit_entry: string
#line 1272 "asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_WordString(output);
	}
#line 2334 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 221: // constlist_32bit_entry: relocexpr_no_str
#line 1284 "asm/parser.y"
                         {
		sect_RelLong(yystack_[0].value.as < Expression > (), 0);
	}
#line 2342 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 222: // constlist_32bit_entry: string
#line 1287 "asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_LongString(output);
	}
#line 2351 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 223: // reloc_8bit: relocexpr
#line 1294 "asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2360 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 224: // reloc_8bit_no_str: relocexpr_no_str
#line 1301 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2369 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 225: // reloc_8bit_offset: "+" relocexpr
#line 1308 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2378 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 226: // reloc_8bit_offset: "-" relocexpr
#line 1312 "asm/parser.y"
                           {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2387 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 227: // reloc_16bit: relocexpr
#line 1319 "asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2396 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 228: // reloc_16bit_no_str: relocexpr_no_str
#line 1326 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2405 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 229: // relocexpr: relocexpr_no_str
#line 1333 "asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2413 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 230: // relocexpr: string
#line 1336 "asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(strToNum(output));
	}
#line 2422 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 231: // relocexpr_no_str: scoped_anon_id
#line 1343 "asm/parser.y"
                       {
		yylhs.value.as < Expression > ().makeSymbol(yystack_[0].value.as < std::string > ());
	}
#line 2430 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 232: // relocexpr_no_str: "number"
#line 1346 "asm/parser.y"
                 {
		yylhs.value.as < Expression > ().makeNumber(yystack_[0].value.as < int32_t > ());
	}
#line 2438 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 233: // relocexpr_no_str: "!" relocexpr
#line 1349 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOGNOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2446 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 234: // relocexpr_no_str: relocexpr "||" relocexpr
#line 1352 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2454 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 235: // relocexpr_no_str: relocexpr "&&" relocexpr
#line 1355 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGAND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2462 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 236: // relocexpr_no_str: relocexpr "==" relocexpr
#line 1358 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGEQ, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2470 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 237: // relocexpr_no_str: relocexpr ">" relocexpr
#line 1361 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2478 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 238: // relocexpr_no_str: relocexpr "<" relocexpr
#line 1364 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2486 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 239: // relocexpr_no_str: relocexpr ">=" relocexpr
#line 1367 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2494 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 240: // relocexpr_no_str: relocexpr "<=" relocexpr
#line 1370 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2502 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 241: // relocexpr_no_str: relocexpr "!=" relocexpr
#line 1373 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGNE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2510 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 242: // relocexpr_no_str: relocexpr "+" relocexpr
#line 1376 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_ADD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2518 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 243: // relocexpr_no_str: relocexpr "-" relocexpr
#line 1379 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SUB, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2526 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 244: // relocexpr_no_str: relocexpr "^" relocexpr
#line 1382 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_XOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2534 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 245: // relocexpr_no_str: relocexpr "|" relocexpr
#line 1385 "asm/parser.y"
                                    {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_OR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2542 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 246: // relocexpr_no_str: relocexpr "&" relocexpr
#line 1388 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_AND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2550 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 247: // relocexpr_no_str: relocexpr "<<" relocexpr
#line 1391 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2558 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 248: // relocexpr_no_str: relocexpr ">>" relocexpr
#line 1394 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2566 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 249: // relocexpr_no_str: relocexpr ">>>" relocexpr
#line 1397 "asm/parser.y"
                                      {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_USHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2574 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 250: // relocexpr_no_str: relocexpr "*" relocexpr
#line 1400 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MUL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2582 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 251: // relocexpr_no_str: relocexpr "/" relocexpr
#line 1403 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_DIV, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2590 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 252: // relocexpr_no_str: relocexpr "%" relocexpr
#line 1406 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MOD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2598 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 253: // relocexpr_no_str: relocexpr "**" relocexpr
#line 1409 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_EXP, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2606 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 254: // relocexpr_no_str: "+" relocexpr
#line 1412 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2614 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 255: // relocexpr_no_str: "-" relocexpr
#line 1415 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2622 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 256: // relocexpr_no_str: "~" relocexpr
#line 1418 "asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2630 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 257: // relocexpr_no_str: "HIGH" "(" relocexpr ")"
#line 1421 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_HIGH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2638 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 258: // relocexpr_no_str: "LOW" "(" relocexpr ")"
#line 1424 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOW, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2646 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 259: // relocexpr_no_str: "BITWIDTH" "(" relocexpr ")"
#line 1427 "asm/parser.y"
                                              {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_BITWIDTH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2654 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 260: // relocexpr_no_str: "TZCOUNT" "(" relocexpr ")"
#line 1430 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_TZCOUNT, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2662 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 261: // relocexpr_no_str: "ISCONST" "(" relocexpr ")"
#line 1433 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeNumber(yystack_[1].value.as < Expression > ().isKnown());
	}
#line 2670 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 262: // relocexpr_no_str: "BANK" "(" scoped_anon_id ")"
#line 1436 "asm/parser.y"
                                               {
		// '@' is also an ID; it is handled here
		yylhs.value.as < Expression > ().makeBankSymbol(yystack_[1].value.as < std::string > ());
	}
#line 2679 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 263: // relocexpr_no_str: "BANK" "(" string ")"
#line 1440 "asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeBankSection(yystack_[1].value.as < std::string > ());
	}
#line 2687 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 264: // relocexpr_no_str: "SIZEOF" "(" string ")"
#line 1443 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeSizeOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2695 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 265: // relocexpr_no_str: "STARTOF" "(" string ")"
#line 1446 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeStartOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2703 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 266: // relocexpr_no_str: "SIZEOF" "(" sect_type ")"
#line 1449 "asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeSizeOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2711 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 267: // relocexpr_no_str: "STARTOF" "(" sect_type ")"
#line 1452 "asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeStartOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2719 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 268: // $@12: %empty
#line 1455 "asm/parser.y"
                 {
		lexer_ToggleStringExpansion(false);
	}
#line 2727 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 269: // relocexpr_no_str: "DEF" $@12 "(" scoped_anon_id ")"
#line 1457 "asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeNumber(sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ()) != nullptr);
		lexer_ToggleStringExpansion(true);
	}
#line 2736 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 270: // relocexpr_no_str: "ROUND" "(" iconst precision_arg ")"
#line 1461 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Round(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2744 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 271: // relocexpr_no_str: "CEIL" "(" iconst precision_arg ")"
#line 1464 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_Ceil(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2752 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 272: // relocexpr_no_str: "FLOOR" "(" iconst precision_arg ")"
#line 1467 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Floor(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2760 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 273: // relocexpr_no_str: "FDIV" "(" iconst "," iconst precision_arg ")"
#line 1470 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Div(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2768 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 274: // relocexpr_no_str: "FMUL" "(" iconst "," iconst precision_arg ")"
#line 1473 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mul(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2776 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 275: // relocexpr_no_str: "FMOD" "(" iconst "," iconst precision_arg ")"
#line 1476 "asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mod(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2784 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 276: // relocexpr_no_str: "POW" "(" iconst "," iconst precision_arg ")"
#line 1479 "asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Pow(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2792 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 277: // relocexpr_no_str: "LOG" "(" iconst "," iconst precision_arg ")"
#line 1482 "asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Log(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2800 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 278: // relocexpr_no_str: "SIN" "(" iconst precision_arg ")"
#line 1485 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Sin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2808 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 279: // relocexpr_no_str: "COS" "(" iconst precision_arg ")"
#line 1488 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Cos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2816 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 280: // relocexpr_no_str: "TAN" "(" iconst precision_arg ")"
#line 1491 "asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Tan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2824 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 281: // relocexpr_no_str: "ASIN" "(" iconst precision_arg ")"
#line 1494 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ASin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2832 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 282: // relocexpr_no_str: "ACOS" "(" iconst precision_arg ")"
#line 1497 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ACos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2840 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 283: // relocexpr_no_str: "ATAN" "(" iconst precision_arg ")"
#line 1500 "asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2848 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 284: // relocexpr_no_str: "ATAN2" "(" iconst "," iconst precision_arg ")"
#line 1503 "asm/parser.y"
                                                                   {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan2(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2856 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 285: // relocexpr_no_str: "STRCMP" "(" string "," string ")"
#line 1506 "asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(yystack_[3].value.as < std::string > ().compare(yystack_[1].value.as < std::string > ()));
	}
#line 2864 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 286: // relocexpr_no_str: "STRIN" "(" string "," string ")"
#line 1509 "asm/parser.y"
                                                     {
		auto pos = yystack_[3].value.as < std::string > ().find(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2874 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 287: // relocexpr_no_str: "STRRIN" "(" string "," string ")"
#line 1514 "asm/parser.y"
                                                      {
		auto pos = yystack_[3].value.as < std::string > ().rfind(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2884 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 288: // relocexpr_no_str: "STRLEN" "(" string ")"
#line 1519 "asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeNumber(strlenUTF8(yystack_[1].value.as < std::string > (), true));
	}
#line 2892 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 289: // relocexpr_no_str: "CHARLEN" "(" string ")"
#line 1522 "asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeNumber(charlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2900 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 290: // relocexpr_no_str: "INCHARMAP" "(" string ")"
#line 1525 "asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeNumber(charmap_HasChar(yystack_[1].value.as < std::string > ()));
	}
#line 2908 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 291: // relocexpr_no_str: "(" relocexpr ")"
#line 1528 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 2916 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 292: // uconst: iconst
#line 1534 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0) {
			fatalerror("Constant must not be negative: %d\n", yylhs.value.as < int32_t > ());
		}
	}
#line 2927 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 293: // iconst: relocexpr
#line 1543 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2935 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 294: // const_no_str: relocexpr_no_str
#line 1549 "asm/parser.y"
                         {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2943 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 295: // precision_arg: %empty
#line 1555 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = fix_Precision();
	}
#line 2951 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 296: // precision_arg: "," iconst
#line 1558 "asm/parser.y"
                       {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 1 || yylhs.value.as < int32_t > () > 31) {
			::error("Fixed-point precision must be between 1 and 31, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = fix_Precision();
		}
	}
#line 2963 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 297: // string: "string"
#line 1568 "asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 2971 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 298: // string: "STRSUB" "(" string "," iconst "," uconst ")"
#line 1571 "asm/parser.y"
                                                                   {
		size_t len = strlenUTF8(yystack_[5].value.as < std::string > (), false);
		uint32_t pos = adjustNegativePos(yystack_[3].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[5].value.as < std::string > (), pos, yystack_[1].value.as < int32_t > ());
	}
#line 2982 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 299: // string: "STRSUB" "(" string "," iconst ")"
#line 1577 "asm/parser.y"
                                                      {
		size_t len = strlenUTF8(yystack_[3].value.as < std::string > (), false);
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[3].value.as < std::string > (), pos, pos > len ? 0 : len + 1 - pos);
	}
#line 2993 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 300: // string: "CHARSUB" "(" string "," iconst ")"
#line 1583 "asm/parser.y"
                                                       {
		size_t len = charlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "CHARSUB");

		yylhs.value.as < std::string > () = charsubUTF8(yystack_[3].value.as < std::string > (), pos);
	}
#line 3004 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 301: // string: "STRCAT" "(" ")"
#line 1589 "asm/parser.y"
                                  {
		yylhs.value.as < std::string > ().clear();
	}
#line 3012 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 302: // string: "STRCAT" "(" strcat_args ")"
#line 1592 "asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3020 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 303: // string: "STRUPR" "(" string ")"
#line 1595 "asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return toupper(c); });
	}
#line 3029 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 304: // string: "STRLWR" "(" string ")"
#line 1599 "asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return tolower(c); });
	}
#line 3038 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 305: // string: "STRRPL" "(" string "," string "," string ")"
#line 1603 "asm/parser.y"
                                                                   {
		yylhs.value.as < std::string > () = strrpl(yystack_[5].value.as < std::string > (), yystack_[3].value.as < std::string > (), yystack_[1].value.as < std::string > ());
	}
#line 3046 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 306: // string: "STRFMT" "(" strfmt_args ")"
#line 1606 "asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = strfmt(yystack_[1].value.as < StrFmtArgList > ().format, yystack_[1].value.as < StrFmtArgList > ().args);
	}
#line 3054 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 307: // string: "SECTION" "(" scoped_anon_id ")"
#line 1609 "asm/parser.y"
                                                   {
		Symbol *sym = sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ());

		if (!sym) {
			if (sym_IsPurgedScoped(yystack_[1].value.as < std::string > ())) {
				fatalerror("Unknown symbol \"%s\"; it was purged\n", yystack_[1].value.as < std::string > ().c_str());
			} else {
				fatalerror("Unknown symbol \"%s\"\n", yystack_[1].value.as < std::string > ().c_str());
			}
		}
		Section const *section = sym->getSection();

		if (!section) {
			fatalerror("\"%s\" does not belong to any section\n", sym->name.c_str());
		}
		// Section names are capped by rgbasm's maximum string length,
		// so this currently can't overflow.
		yylhs.value.as < std::string > () = section->name;
	}
#line 3078 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 308: // strcat_args: string
#line 1631 "asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 3086 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 309: // strcat_args: strcat_args "," string
#line 1634 "asm/parser.y"
                                   {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		yylhs.value.as < std::string > ().append(yystack_[0].value.as < std::string > ());
	}
#line 3095 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 310: // strfmt_args: %empty
#line 1641 "asm/parser.y"
                 {}
#line 3101 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 311: // strfmt_args: string strfmt_va_args
#line 1642 "asm/parser.y"
                                {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[0].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().format = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3110 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 312: // strfmt_va_args: %empty
#line 1649 "asm/parser.y"
                 {}
#line 3116 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 313: // strfmt_va_args: strfmt_va_args "," const_no_str
#line 1650 "asm/parser.y"
                                            {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(static_cast<uint32_t>(yystack_[0].value.as < int32_t > ()));
	}
#line 3125 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 314: // strfmt_va_args: strfmt_va_args "," string
#line 1654 "asm/parser.y"
                                      {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(std::move(yystack_[0].value.as < std::string > ()));
	}
#line 3134 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 315: // section: "SECTION" sect_mod string "," sect_type sect_org sect_attrs
#line 1661 "asm/parser.y"
                                                                        {
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3142 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 316: // pushs_section: "PUSHS" sect_mod string "," sect_type sect_org sect_attrs
#line 1667 "asm/parser.y"
                                                                      {
		sect_PushSection();
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3151 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 317: // sect_mod: %empty
#line 1674 "asm/parser.y"
               {
		yylhs.value.as < SectionModifier > () = SECTION_NORMAL;
	}
#line 3159 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 318: // sect_mod: "UNION"
#line 1677 "asm/parser.y"
                    {
		yylhs.value.as < SectionModifier > () = SECTION_UNION;
	}
#line 3167 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 319: // sect_mod: "FRAGMENT"
#line 1680 "asm/parser.y"
                       {
		yylhs.value.as < SectionModifier > () = SECTION_FRAGMENT;
	}
#line 3175 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 320: // sect_type: "WRAM0"
#line 1686 "asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAM0;
	}
#line 3183 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 321: // sect_type: "VRAM"
#line 1689 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_VRAM;
	}
#line 3191 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 322: // sect_type: "ROMX"
#line 1692 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROMX;
	}
#line 3199 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 323: // sect_type: "ROM0"
#line 1695 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROM0;
	}
#line 3207 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 324: // sect_type: "HRAM"
#line 1698 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_HRAM;
	}
#line 3215 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 325: // sect_type: "WRAMX"
#line 1701 "asm/parser.y"
                     {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAMX;
	}
#line 3223 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 326: // sect_type: "SRAM"
#line 1704 "asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_SRAM;
	}
#line 3231 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 327: // sect_type: "OAM"
#line 1707 "asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_OAM;
	}
#line 3239 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 328: // sect_org: %empty
#line 1713 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = -1;
	}
#line 3247 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 329: // sect_org: "[" uconst "]"
#line 1716 "asm/parser.y"
                               {
		yylhs.value.as < int32_t > () = yystack_[1].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 0xFFFF) {
			::error("Address $%x is not 16-bit\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = -1;
		}
	}
#line 3259 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 330: // sect_attrs: %empty
#line 1726 "asm/parser.y"
               {
		yylhs.value.as < SectionSpec > ().alignment = 0;
		yylhs.value.as < SectionSpec > ().alignOfs = 0;
		yylhs.value.as < SectionSpec > ().bank = -1;
	}
#line 3269 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 331: // sect_attrs: sect_attrs "," "ALIGN" "[" align_spec "]"
#line 1731 "asm/parser.y"
                                                              {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().alignment = yystack_[1].value.as < AlignmentSpec > ().alignment;
		yylhs.value.as < SectionSpec > ().alignOfs = yystack_[1].value.as < AlignmentSpec > ().alignOfs;
	}
#line 3279 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 332: // sect_attrs: sect_attrs "," "BANK" "[" uconst "]"
#line 1736 "asm/parser.y"
                                                        {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().bank = yystack_[1].value.as < int32_t > (); // We cannot check the validity of this yet
	}
#line 3288 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 381: // sm83_adc: "adc" op_a_n
#line 1799 "asm/parser.y"
                        {
		sect_ConstByte(0xCE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3297 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 382: // sm83_adc: "adc" op_a_r
#line 1803 "asm/parser.y"
                          {
		sect_ConstByte(0x88 | yystack_[0].value.as < int32_t > ());
	}
#line 3305 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 383: // sm83_add: "add" op_a_n
#line 1809 "asm/parser.y"
                        {
		sect_ConstByte(0xC6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3314 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 384: // sm83_add: "add" op_a_r
#line 1813 "asm/parser.y"
                          {
		sect_ConstByte(0x80 | yystack_[0].value.as < int32_t > ());
	}
#line 3322 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 385: // sm83_add: "add" "hl" "," reg_ss
#line 1816 "asm/parser.y"
                                        {
		sect_ConstByte(0x09 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3330 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 386: // sm83_add: "add" "sp" "," reloc_8bit
#line 1819 "asm/parser.y"
                                            {
		sect_ConstByte(0xE8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3339 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 387: // sm83_and: "and" op_a_n
#line 1826 "asm/parser.y"
                        {
		sect_ConstByte(0xE6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3348 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 388: // sm83_and: "and" op_a_r
#line 1830 "asm/parser.y"
                          {
		sect_ConstByte(0xA0 | yystack_[0].value.as < int32_t > ());
	}
#line 3356 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 389: // sm83_bit: "bit" bit_const "," reg_r
#line 1836 "asm/parser.y"
                                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3365 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 390: // sm83_call: "call" reloc_16bit
#line 1843 "asm/parser.y"
                              {
		sect_ConstByte(0xCD);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3374 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 391: // sm83_call: "call" ccode_expr "," reloc_16bit
#line 1847 "asm/parser.y"
                                                 {
		sect_ConstByte(0xC4 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3383 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 392: // sm83_ccf: "ccf"
#line 1854 "asm/parser.y"
                 {
		sect_ConstByte(0x3F);
	}
#line 3391 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 393: // sm83_cp: "cp" op_a_n
#line 1860 "asm/parser.y"
                       {
		sect_ConstByte(0xFE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3400 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 394: // sm83_cp: "cp" op_a_r
#line 1864 "asm/parser.y"
                         {
		sect_ConstByte(0xB8 | yystack_[0].value.as < int32_t > ());
	}
#line 3408 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 395: // sm83_cpl: "cpl"
#line 1870 "asm/parser.y"
                 {
		sect_ConstByte(0x2F);
	}
#line 3416 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 396: // sm83_cpl: "cpl" MODE_A
#line 1873 "asm/parser.y"
                          {
		sect_ConstByte(0x2F);
	}
#line 3424 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 397: // sm83_daa: "daa"
#line 1879 "asm/parser.y"
                 {
		sect_ConstByte(0x27);
	}
#line 3432 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 398: // sm83_dec: "dec" reg_r
#line 1885 "asm/parser.y"
                       {
		sect_ConstByte(0x05 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3440 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 399: // sm83_dec: "dec" reg_ss
#line 1888 "asm/parser.y"
                          {
		sect_ConstByte(0x0B | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3448 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 400: // sm83_di: "di"
#line 1894 "asm/parser.y"
                {
		sect_ConstByte(0xF3);
	}
#line 3456 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 401: // sm83_ei: "ei"
#line 1900 "asm/parser.y"
                {
		sect_ConstByte(0xFB);
	}
#line 3464 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 402: // sm83_halt: "halt"
#line 1906 "asm/parser.y"
                  {
		sect_ConstByte(0x76);
	}
#line 3472 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 403: // sm83_inc: "inc" reg_r
#line 1912 "asm/parser.y"
                       {
		sect_ConstByte(0x04 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3480 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 404: // sm83_inc: "inc" reg_ss
#line 1915 "asm/parser.y"
                          {
		sect_ConstByte(0x03 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3488 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 405: // sm83_jp: "jp" reloc_16bit
#line 1921 "asm/parser.y"
                            {
		sect_ConstByte(0xC3);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3497 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 406: // sm83_jp: "jp" ccode_expr "," reloc_16bit
#line 1925 "asm/parser.y"
                                               {
		sect_ConstByte(0xC2 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3506 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 407: // sm83_jp: "jp" "hl"
#line 1929 "asm/parser.y"
                          {
		sect_ConstByte(0xE9);
	}
#line 3514 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 408: // sm83_jr: "jr" reloc_16bit
#line 1935 "asm/parser.y"
                            {
		sect_ConstByte(0x18);
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3523 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 409: // sm83_jr: "jr" ccode_expr "," reloc_16bit
#line 1939 "asm/parser.y"
                                               {
		sect_ConstByte(0x20 | (yystack_[2].value.as < int32_t > () << 3));
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3532 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 410: // sm83_ldi: "ldi" "[" "hl" "]" "," MODE_A
#line 1946 "asm/parser.y"
                                                    {
		sect_ConstByte(0x02 | (2 << 4));
	}
#line 3540 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 411: // sm83_ldi: "ldi" MODE_A "," "[" "hl" "]"
#line 1949 "asm/parser.y"
                                                      {
		sect_ConstByte(0x0A | (2 << 4));
	}
#line 3548 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 412: // sm83_ldd: "ldd" "[" "hl" "]" "," MODE_A
#line 1955 "asm/parser.y"
                                                    {
		sect_ConstByte(0x02 | (3 << 4));
	}
#line 3556 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 413: // sm83_ldd: "ldd" MODE_A "," "[" "hl" "]"
#line 1958 "asm/parser.y"
                                                      {
		sect_ConstByte(0x0A | (3 << 4));
	}
#line 3564 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 414: // sm83_ldh: "ldh" MODE_A "," op_mem_ind
#line 1964 "asm/parser.y"
                                         {
		if (yystack_[0].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xF0);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3580 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 415: // sm83_ldh: "ldh" op_mem_ind "," MODE_A
#line 1975 "asm/parser.y"
                                           {
		if (yystack_[2].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xE0);
		sect_RelByte(yystack_[2].value.as < Expression > (), 1);
	}
#line 3596 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 416: // sm83_ldh: "ldh" MODE_A "," c_ind
#line 1986 "asm/parser.y"
                                      {
		sect_ConstByte(0xF2);
	}
#line 3604 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 417: // sm83_ldh: "ldh" MODE_A "," ff00_c_ind
#line 1989 "asm/parser.y"
                                           {
		sect_ConstByte(0xF2);
	}
#line 3612 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 418: // sm83_ldh: "ldh" c_ind "," MODE_A
#line 1992 "asm/parser.y"
                                      {
		sect_ConstByte(0xE2);
	}
#line 3620 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 419: // sm83_ldh: "ldh" ff00_c_ind "," MODE_A
#line 1995 "asm/parser.y"
                                           {
		sect_ConstByte(0xE2);
	}
#line 3628 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 421: // ff00_c_ind: "[" relocexpr "+" MODE_C "]"
#line 2003 "asm/parser.y"
                                              {
		// This has to use `relocexpr`, not `iconst`, to avoid a shift/reduce conflict
		if (yystack_[3].value.as < Expression > ().getConstVal() != 0xFF00) {
			::error("Base value must be equal to $FF00 for $FF00+C\n");
		}
	}
#line 3639 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 430: // sm83_ld_hl: "ld" "hl" "," "sp" reloc_8bit_offset
#line 2023 "asm/parser.y"
                                                        {
		sect_ConstByte(0xF8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3648 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 431: // sm83_ld_hl: "ld" "hl" "," reloc_16bit
#line 2027 "asm/parser.y"
                                            {
		sect_ConstByte(0x01 | (REG_HL << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3657 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 432: // sm83_ld_sp: "ld" "sp" "," "hl"
#line 2034 "asm/parser.y"
                                      {
		sect_ConstByte(0xF9);
	}
#line 3665 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 433: // sm83_ld_sp: "ld" "sp" "," reloc_16bit
#line 2037 "asm/parser.y"
                                            {
		sect_ConstByte(0x01 | (REG_SP << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3674 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 434: // sm83_ld_mem: "ld" op_mem_ind "," "sp"
#line 2044 "asm/parser.y"
                                         {
		sect_ConstByte(0x08);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3683 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 435: // sm83_ld_mem: "ld" op_mem_ind "," MODE_A
#line 2048 "asm/parser.y"
                                          {
		sect_ConstByte(0xEA);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3692 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 436: // sm83_ld_c_ind: "ld" ff00_c_ind "," MODE_A
#line 2055 "asm/parser.y"
                                        {
		sect_ConstByte(0xE2);
	}
#line 3700 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 437: // sm83_ld_c_ind: "ld" c_ind "," MODE_A
#line 2058 "asm/parser.y"
                                     {
		warning(WARNING_OBSOLETE, "LD [C], A is deprecated; use LDH [C], A\n");
		sect_ConstByte(0xE2);
	}
#line 3709 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 438: // sm83_ld_rr: "ld" reg_rr "," MODE_A
#line 2065 "asm/parser.y"
                                    {
		sect_ConstByte(0x02 | (yystack_[2].value.as < int32_t > () << 4));
	}
#line 3717 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 439: // sm83_ld_r_no_a: "ld" reg_r_no_a "," reloc_8bit
#line 2071 "asm/parser.y"
                                            {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3726 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 440: // sm83_ld_r_no_a: "ld" reg_r_no_a "," reg_r
#line 2075 "asm/parser.y"
                                         {
		if (yystack_[2].value.as < int32_t > () == REG_HL_IND && yystack_[0].value.as < int32_t > () == REG_HL_IND) {
			::error("LD [HL], [HL] is not a valid instruction\n");
		} else {
			sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
		}
	}
#line 3738 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 441: // sm83_ld_a: "ld" reg_a "," reloc_8bit
#line 2085 "asm/parser.y"
                                       {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3747 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 442: // sm83_ld_a: "ld" reg_a "," reg_r
#line 2089 "asm/parser.y"
                                    {
		sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3755 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 443: // sm83_ld_a: "ld" reg_a "," ff00_c_ind
#line 2092 "asm/parser.y"
                                         {
		sect_ConstByte(0xF2);
	}
#line 3763 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 444: // sm83_ld_a: "ld" reg_a "," c_ind
#line 2095 "asm/parser.y"
                                    {
		warning(WARNING_OBSOLETE, "LD A, [C] is deprecated; use LDH A, [C]\n");
		sect_ConstByte(0xF2);
	}
#line 3772 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 445: // sm83_ld_a: "ld" reg_a "," reg_rr
#line 2099 "asm/parser.y"
                                     {
		sect_ConstByte(0x0A | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3780 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 446: // sm83_ld_a: "ld" reg_a "," op_mem_ind
#line 2102 "asm/parser.y"
                                         {
		sect_ConstByte(0xFA);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3789 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 447: // sm83_ld_ss: "ld" "bc" "," reloc_16bit
#line 2109 "asm/parser.y"
                                          {
		sect_ConstByte(0x01 | (REG_BC << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3798 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 448: // sm83_ld_ss: "ld" "de" "," reloc_16bit
#line 2113 "asm/parser.y"
                                            {
		sect_ConstByte(0x01 | (REG_DE << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3807 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 449: // sm83_nop: "nop"
#line 2122 "asm/parser.y"
                 {
		sect_ConstByte(0x00);
	}
#line 3815 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 450: // sm83_or: "or" op_a_n
#line 2128 "asm/parser.y"
                       {
		sect_ConstByte(0xF6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3824 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 451: // sm83_or: "or" op_a_r
#line 2132 "asm/parser.y"
                         {
		sect_ConstByte(0xB0 | yystack_[0].value.as < int32_t > ());
	}
#line 3832 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 452: // sm83_pop: "pop" reg_tt
#line 2138 "asm/parser.y"
                        {
		sect_ConstByte(0xC1 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3840 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 453: // sm83_push: "push" reg_tt
#line 2144 "asm/parser.y"
                         {
		sect_ConstByte(0xC5 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3848 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 454: // sm83_res: "res" bit_const "," reg_r
#line 2150 "asm/parser.y"
                                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x80 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3857 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 455: // sm83_ret: "ret"
#line 2157 "asm/parser.y"
                 {
		sect_ConstByte(0xC9);
	}
#line 3865 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 456: // sm83_ret: "ret" ccode_expr
#line 2160 "asm/parser.y"
                              {
		sect_ConstByte(0xC0 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3873 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 457: // sm83_reti: "reti"
#line 2166 "asm/parser.y"
                  {
		sect_ConstByte(0xD9);
	}
#line 3881 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 458: // sm83_rl: "rl" reg_r
#line 2172 "asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x10 | yystack_[0].value.as < int32_t > ());
	}
#line 3890 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 459: // sm83_rla: "rla"
#line 2179 "asm/parser.y"
                 {
		sect_ConstByte(0x17);
	}
#line 3898 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 460: // sm83_rlc: "rlc" reg_r
#line 2185 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x00 | yystack_[0].value.as < int32_t > ());
	}
#line 3907 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 461: // sm83_rlca: "rlca"
#line 2192 "asm/parser.y"
                  {
		sect_ConstByte(0x07);
	}
#line 3915 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 462: // sm83_rr: "rr" reg_r
#line 2198 "asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x18 | yystack_[0].value.as < int32_t > ());
	}
#line 3924 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 463: // sm83_rra: "rra"
#line 2205 "asm/parser.y"
                 {
		sect_ConstByte(0x1F);
	}
#line 3932 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 464: // sm83_rrc: "rrc" reg_r
#line 2211 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x08 | yystack_[0].value.as < int32_t > ());
	}
#line 3941 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 465: // sm83_rrca: "rrca"
#line 2218 "asm/parser.y"
                  {
		sect_ConstByte(0x0F);
	}
#line 3949 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 466: // sm83_rst: "rst" reloc_8bit
#line 2224 "asm/parser.y"
                            {
		yystack_[0].value.as < Expression > ().makeCheckRST();
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			sect_RelByte(yystack_[0].value.as < Expression > (), 0);
		} else {
			sect_ConstByte(0xC7 | yystack_[0].value.as < Expression > ().value());
		}
	}
#line 3962 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 467: // sm83_sbc: "sbc" op_a_n
#line 2235 "asm/parser.y"
                        {
		sect_ConstByte(0xDE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3971 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 468: // sm83_sbc: "sbc" op_a_r
#line 2239 "asm/parser.y"
                          {
		sect_ConstByte(0x98 | yystack_[0].value.as < int32_t > ());
	}
#line 3979 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 469: // sm83_scf: "scf"
#line 2245 "asm/parser.y"
                 {
		sect_ConstByte(0x37);
	}
#line 3987 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 470: // sm83_set: "set" bit_const "," reg_r
#line 2251 "asm/parser.y"
                                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0xC0 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3996 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 471: // sm83_sla: "sla" reg_r
#line 2258 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x20 | yystack_[0].value.as < int32_t > ());
	}
#line 4005 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 472: // sm83_sra: "sra" reg_r
#line 2265 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x28 | yystack_[0].value.as < int32_t > ());
	}
#line 4014 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 473: // sm83_srl: "srl" reg_r
#line 2272 "asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x38 | yystack_[0].value.as < int32_t > ());
	}
#line 4023 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 474: // sm83_stop: "stop"
#line 2279 "asm/parser.y"
                  {
		sect_ConstByte(0x10);
		sect_ConstByte(0x00);
	}
#line 4032 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 475: // sm83_stop: "stop" reloc_8bit
#line 2283 "asm/parser.y"
                               {
		sect_ConstByte(0x10);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4041 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 476: // sm83_sub: "sub" op_a_n
#line 2290 "asm/parser.y"
                        {
		sect_ConstByte(0xD6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4050 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 477: // sm83_sub: "sub" op_a_r
#line 2294 "asm/parser.y"
                          {
		sect_ConstByte(0x90 | yystack_[0].value.as < int32_t > ());
	}
#line 4058 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 478: // sm83_swap: "swap" reg_r
#line 2300 "asm/parser.y"
                        {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x30 | yystack_[0].value.as < int32_t > ());
	}
#line 4067 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 479: // sm83_xor: "xor" op_a_n
#line 2307 "asm/parser.y"
                        {
		sect_ConstByte(0xEE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4076 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 480: // sm83_xor: "xor" op_a_r
#line 2311 "asm/parser.y"
                          {
		sect_ConstByte(0xA8 | yystack_[0].value.as < int32_t > ());
	}
#line 4084 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 481: // op_mem_ind: "[" reloc_16bit "]"
#line 2319 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 4092 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 482: // op_a_r: reg_r
#line 2325 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4098 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 483: // op_a_r: MODE_A "," reg_r
#line 2326 "asm/parser.y"
                             {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
	}
#line 4106 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 484: // op_a_n: reloc_8bit
#line 2332 "asm/parser.y"
                   {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4114 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 485: // op_a_n: MODE_A "," reloc_8bit
#line 2335 "asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4122 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 500: // ccode_expr: ccode
#line 2378 "asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4128 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 501: // ccode_expr: "!" ccode_expr
#line 2379 "asm/parser.y"
                                 {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > () ^ 1;
	}
#line 4136 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 502: // ccode: "nz"
#line 2385 "asm/parser.y"
              {
		yylhs.value.as < int32_t > () = CC_NZ;
	}
#line 4144 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 503: // ccode: "z"
#line 2388 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = CC_Z;
	}
#line 4152 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 504: // ccode: "nc"
#line 2391 "asm/parser.y"
                {
		yylhs.value.as < int32_t > () = CC_NC;
	}
#line 4160 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 505: // ccode: "c"
#line 2394 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = CC_C;
	}
#line 4168 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 506: // reg_r: reg_r_no_a
#line 2399 "asm/parser.y"
       { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4174 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 507: // reg_r: reg_a
#line 2399 "asm/parser.y"
                    { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4180 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 508: // reg_r_no_a: MODE_B
#line 2402 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_B;
	}
#line 4188 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 509: // reg_r_no_a: MODE_C
#line 2405 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_C;
	}
#line 4196 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 510: // reg_r_no_a: MODE_D
#line 2408 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_D;
	}
#line 4204 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 511: // reg_r_no_a: MODE_E
#line 2411 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_E;
	}
#line 4212 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 512: // reg_r_no_a: MODE_H
#line 2414 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_H;
	}
#line 4220 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 513: // reg_r_no_a: MODE_L
#line 2417 "asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_L;
	}
#line 4228 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 514: // reg_r_no_a: "[" "hl" "]"
#line 2420 "asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_HL_IND;
	}
#line 4236 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 515: // reg_a: MODE_A
#line 2426 "asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_A;
	}
#line 4244 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 516: // reg_tt: "bc"
#line 2432 "asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4252 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 517: // reg_tt: "de"
#line 2435 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4260 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 518: // reg_tt: "hl"
#line 2438 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4268 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 519: // reg_tt: "af"
#line 2441 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_AF;
	}
#line 4276 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 520: // reg_ss: "bc"
#line 2447 "asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4284 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 521: // reg_ss: "de"
#line 2450 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4292 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 522: // reg_ss: "hl"
#line 2453 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4300 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 523: // reg_ss: "sp"
#line 2456 "asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_SP;
	}
#line 4308 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 524: // reg_rr: "[" "bc" "]"
#line 2462 "asm/parser.y"
                              {
		yylhs.value.as < int32_t > () = REG_BC_IND;
	}
#line 4316 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 525: // reg_rr: "[" "de" "]"
#line 2465 "asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_DE_IND;
	}
#line 4324 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 526: // reg_rr: hl_ind_inc
#line 2468 "asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDINC;
	}
#line 4332 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;

  case 527: // reg_rr: hl_ind_dec
#line 2471 "asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDDEC;
	}
#line 4340 "/build/rgbds/src/rgbds/src/asm/parser.cpp"
    break;


#line 4344 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

            default:
              break;
            }
        }
#if YY_EXCEPTIONS
      catch (const syntax_error& yyexc)
        {
          YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
          error (yyexc);
          YYERROR;
        }
#endif // YY_EXCEPTIONS
      YY_SYMBOL_PRINT ("-> $$ =", yylhs);
      yypop_ (yylen);
      yylen = 0;

      // Shift the result of the reduction.
      yypush_ (YY_NULLPTR, YY_MOVE (yylhs));
    }
    goto yynewstate;


  /*--------------------------------------.
  | yyerrlab -- here on detecting error.  |
  `--------------------------------------*/
  yyerrlab:
    // If not already recovering from an error, report this error.
    if (!yyerrstatus_)
      {
        ++yynerrs_;
        context yyctx (*this, yyla);
        std::string msg = yysyntax_error_ (yyctx);
        error (YY_MOVE (msg));
      }


    if (yyerrstatus_ == 3)
      {
        /* If just tried and failed to reuse lookahead token after an
           error, discard it.  */

        // Return failure if at end of input.
        if (yyla.kind () == symbol_kind::S_YYEOF)
          YYABORT;
        else if (!yyla.empty ())
          {
            yy_destroy_ ("Error: discarding", yyla);
            yyla.clear ();
          }
      }

    // Else will try to reuse lookahead token after shifting the error token.
    goto yyerrlab1;


  /*---------------------------------------------------.
  | yyerrorlab -- error raised explicitly by YYERROR.  |
  `---------------------------------------------------*/
  yyerrorlab:
    /* Pacify compilers when the user code never invokes YYERROR and
       the label yyerrorlab therefore never appears in user code.  */
    if (false)
      YYERROR;

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYERROR.  */
    yypop_ (yylen);
    yylen = 0;
    YY_STACK_PRINT ();
    goto yyerrlab1;


  /*-------------------------------------------------------------.
  | yyerrlab1 -- common code for both syntax error and YYERROR.  |
  `-------------------------------------------------------------*/
  yyerrlab1:
    yyerrstatus_ = 3;   // Each real token shifted decrements this.
    // Pop stack until we find a state that shifts the error token.
    for (;;)
      {
        yyn = yypact_[+yystack_[0].state];
        if (!yy_pact_value_is_default_ (yyn))
          {
            yyn += symbol_kind::S_YYerror;
            if (0 <= yyn && yyn <= yylast_
                && yycheck_[yyn] == symbol_kind::S_YYerror)
              {
                yyn = yytable_[yyn];
                if (0 < yyn)
                  break;
              }
          }

        // Pop the current state because it cannot handle the error token.
        if (yystack_.size () == 1)
          YYABORT;

        yy_destroy_ ("Error: popping", yystack_[0]);
        yypop_ ();
        YY_STACK_PRINT ();
      }
    {
      stack_symbol_type error_token;


      // Shift the error token.
      yy_lac_discard_ ("error recovery");
      error_token.state = state_type (yyn);
      yypush_ ("Shifting", YY_MOVE (error_token));
    }
    goto yynewstate;


  /*-------------------------------------.
  | yyacceptlab -- YYACCEPT comes here.  |
  `-------------------------------------*/
  yyacceptlab:
    yyresult = 0;
    goto yyreturn;


  /*-----------------------------------.
  | yyabortlab -- YYABORT comes here.  |
  `-----------------------------------*/
  yyabortlab:
    yyresult = 1;
    goto yyreturn;


  /*-----------------------------------------------------.
  | yyreturn -- parsing is finished, return the result.  |
  `-----------------------------------------------------*/
  yyreturn:
    if (!yyla.empty ())
      yy_destroy_ ("Cleanup: discarding lookahead", yyla);

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYABORT or YYACCEPT.  */
    yypop_ (yylen);
    YY_STACK_PRINT ();
    while (1 < yystack_.size ())
      {
        yy_destroy_ ("Cleanup: popping", yystack_[0]);
        yypop_ ();
      }

    return yyresult;
  }
#if YY_EXCEPTIONS
    catch (...)
      {
        YYCDEBUG << "Exception caught: cleaning lookahead and stack\n";
        // Do not try to display the values of the reclaimed symbols,
        // as their printers might throw an exception.
        if (!yyla.empty ())
          yy_destroy_ (YY_NULLPTR, yyla);

        while (1 < yystack_.size ())
          {
            yy_destroy_ (YY_NULLPTR, yystack_[0]);
            yypop_ ();
          }
        throw;
      }
#endif // YY_EXCEPTIONS
  }

  void
  parser::error (const syntax_error& yyexc)
  {
    error (yyexc.what ());
  }

  const char *
  parser::symbol_name (symbol_kind_type yysymbol)
  {
    static const char *const yy_sname[] =
    {
    "end of file", "error", "invalid token", "end of line", "end of buffer",
  ",", ":", "::", "[", "]", "(", ")", "+", "-", "*", "/", "%", "**", "==",
  "!=", "<", ">", "<=", ">=", "&&", "||", "!", "&", "|", "^", "<<", ">>",
  ">>>", "~", "NEG", "=", "+=", "-=", "*=", "/=", "%=", "&=", "|=", "^=",
  "<<=", ">>=", "a", "b", "c", "d", "e", "h", "l", "af", "bc", "de", "hl",
  "sp", "hli/hl+", "hld/hl-", "z", "nz", "nc", "adc", "add", "and", "bit",
  "call", "ccf", "cp", "cpl", "daa", "dec", "di", "ei", "halt", "inc",
  "jp", "jr", "ldd", "ldh", "ldi", "ld", "nop", "or", "pop", "push", "res",
  "reti", "ret", "rla", "rlca", "rlc", "rl", "rra", "rrca", "rrc", "rr",
  "rst", "sbc", "scf", "set", "sla", "sra", "srl", "stop", "sub", "swap",
  "xor", "ALIGN", "ASSERT", "BREAK", "CHARMAP", "DB", "DL", "DS", "DW",
  "ELIF", "ELSE", "ENDC", "ENDL", "ENDM", "ENDR", "ENDSECTION", "ENDU",
  "EQU", "EQUS", "EXPORT", "FAIL", "FATAL", "FOR", "FRAGMENT", "IF",
  "INCBIN", "INCLUDE", "LOAD", "MACRO", "NEWCHARMAP", "NEXTU", "OPT",
  "POPC", "POPO", "POPS", "PRINTLN", "PRINT", "PURGE", "PUSHC", "PUSHO",
  "PUSHS", "RB", "REDEF", "REPT", "RSRESET", "RSSET", "RW", "SECTION",
  "SETCHARMAP", "SHIFT", "STATIC_ASSERT", "UNION", "WARN", "ACOS", "ASIN",
  "ATAN", "ATAN2", "BANK", "BITWIDTH", "CEIL", "CHARLEN", "CHARSUB", "COS",
  "DEF", "FDIV", "FLOOR", "FMOD", "FMUL", "HIGH", "INCHARMAP", "ISCONST",
  "LOG", "LOW", "POW", "ROUND", "SIN", "SIZEOF", "STARTOF", "STRCAT",
  "STRCMP", "STRFMT", "STRIN", "STRLEN", "STRLWR", "STRRIN", "STRRPL",
  "STRSUB", "STRUPR", "TAN", "TZCOUNT", "HRAM", "OAM", "ROM0", "ROMX",
  "SRAM", "VRAM", "WRAM0", "WRAMX", "number", "string", "label",
  "identifier", "local identifier", "anonymous label", "$accept",
  "asm_file", "lines", "$@1", "diff_mark", "line", "endofline",
  "line_directive", "if", "elif", "else", "plain_directive", "endc",
  "def_id", "$@2", "redef_id", "$@3", "scoped_id", "scoped_anon_id",
  "label", "macro", "$@4", "macro_args", "directive", "def_numeric",
  "trailing_comma", "compound_eq", "align", "align_spec", "opt", "$@5",
  "opt_list", "opt_list_entry", "popo", "pusho", "$@6", "pusho_opt_list",
  "pops", "pushs", "endsection", "fail", "warn", "assert_type", "assert",
  "shift", "shift_const", "load", "rept", "for", "$@7", "$@8",
  "capture_rept", "for_args", "break", "def_macro", "$@9", "$@10",
  "capture_macro", "rsset", "rsreset", "rs_uconst", "union", "nextu",
  "endu", "ds", "ds_args", "db", "dw", "dl", "def_equ", "redef_equ",
  "def_set", "def_rb", "def_rw", "def_rl", "def_equs", "redef_equs",
  "purge", "$@11", "purge_args", "export", "export_list",
  "export_list_entry", "export_def", "include", "incbin", "charmap",
  "charmap_args", "newcharmap", "setcharmap", "pushc", "pushc_setcharmap",
  "popc", "print", "println", "print_exprs", "print_expr", "bit_const",
  "constlist_8bit", "constlist_8bit_entry", "constlist_16bit",
  "constlist_16bit_entry", "constlist_32bit", "constlist_32bit_entry",
  "reloc_8bit", "reloc_8bit_no_str", "reloc_8bit_offset", "reloc_16bit",
  "reloc_16bit_no_str", "relocexpr", "relocexpr_no_str", "$@12", "uconst",
  "iconst", "const_no_str", "precision_arg", "string", "strcat_args",
  "strfmt_args", "strfmt_va_args", "section", "pushs_section", "sect_mod",
  "sect_type", "sect_org", "sect_attrs", "cpu_commands", "cpu_command",
  "sm83_adc", "sm83_add", "sm83_and", "sm83_bit", "sm83_call", "sm83_ccf",
  "sm83_cp", "sm83_cpl", "sm83_daa", "sm83_dec", "sm83_di", "sm83_ei",
  "sm83_halt", "sm83_inc", "sm83_jp", "sm83_jr", "sm83_ldi", "sm83_ldd",
  "sm83_ldh", "c_ind", "ff00_c_ind", "sm83_ld", "sm83_ld_hl", "sm83_ld_sp",
  "sm83_ld_mem", "sm83_ld_c_ind", "sm83_ld_rr", "sm83_ld_r_no_a",
  "sm83_ld_a", "sm83_ld_ss", "sm83_nop", "sm83_or", "sm83_pop",
  "sm83_push", "sm83_res", "sm83_ret", "sm83_reti", "sm83_rl", "sm83_rla",
  "sm83_rlc", "sm83_rlca", "sm83_rr", "sm83_rra", "sm83_rrc", "sm83_rrca",
  "sm83_rst", "sm83_sbc", "sm83_scf", "sm83_set", "sm83_sla", "sm83_sra",
  "sm83_srl", "sm83_stop", "sm83_sub", "sm83_swap", "sm83_xor",
  "op_mem_ind", "op_a_r", "op_a_n", "MODE_A", "MODE_B", "MODE_C", "MODE_D",
  "MODE_E", "MODE_H", "MODE_L", "ccode_expr", "ccode", "reg_r",
  "reg_r_no_a", "reg_a", "reg_tt", "reg_ss", "reg_rr", "hl_ind_inc",
  "hl_ind_dec", YY_NULLPTR
    };
    return yy_sname[yysymbol];
  }



  // parser::context.
  parser::context::context (const parser& yyparser, const symbol_type& yyla)
    : yyparser_ (yyparser)
    , yyla_ (yyla)
  {}

  int
  parser::context::expected_tokens (symbol_kind_type yyarg[], int yyargn) const
  {
    // Actual number of expected tokens
    int yycount = 0;

#if YYDEBUG
    // Execute LAC once. We don't care if it is successful, we
    // only do it for the sake of debugging output.
    if (!yyparser_.yy_lac_established_)
      yyparser_.yy_lac_check_ (yyla_.kind ());
#endif

    for (int yyx = 0; yyx < YYNTOKENS; ++yyx)
      {
        symbol_kind_type yysym = YY_CAST (symbol_kind_type, yyx);
        if (yysym != symbol_kind::S_YYerror
            && yysym != symbol_kind::S_YYUNDEF
            && yyparser_.yy_lac_check_ (yysym))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = yysym;
          }
      }
    if (yyarg && yycount == 0 && 0 < yyargn)
      yyarg[0] = symbol_kind::S_YYEMPTY;
    return yycount;
  }




  bool
  parser::yy_lac_check_ (symbol_kind_type yytoken) const
  {
    // Logically, the yylac_stack's lifetime is confined to this function.
    // Clear it, to get rid of potential left-overs from previous call.
    yylac_stack_.clear ();
    // Reduce until we encounter a shift and thereby accept the token.
#if YYDEBUG
    YYCDEBUG << "LAC: checking lookahead " << symbol_name (yytoken) << ':';
#endif
    std::ptrdiff_t lac_top = 0;
    while (true)
      {
        state_type top_state = (yylac_stack_.empty ()
                                ? yystack_[lac_top].state
                                : yylac_stack_.back ());
        int yyrule = yypact_[+top_state];
        if (yy_pact_value_is_default_ (yyrule)
            || (yyrule += yytoken) < 0 || yylast_ < yyrule
            || yycheck_[yyrule] != yytoken)
          {
            // Use the default action.
            yyrule = yydefact_[+top_state];
            if (yyrule == 0)
              {
                YYCDEBUG << " Err\n";
                return false;
              }
          }
        else
          {
            // Use the action from yytable.
            yyrule = yytable_[yyrule];
            if (yy_table_value_is_error_ (yyrule))
              {
                YYCDEBUG << " Err\n";
                return false;
              }
            if (0 < yyrule)
              {
                YYCDEBUG << " S" << yyrule << '\n';
                return true;
              }
            yyrule = -yyrule;
          }
        // By now we know we have to simulate a reduce.
        YYCDEBUG << " R" << yyrule - 1;
        // Pop the corresponding number of values from the stack.
        {
          std::ptrdiff_t yylen = yyr2_[yyrule];
          // First pop from the LAC stack as many tokens as possible.
          std::ptrdiff_t lac_size = std::ptrdiff_t (yylac_stack_.size ());
          if (yylen < lac_size)
            {
              yylac_stack_.resize (std::size_t (lac_size - yylen));
              yylen = 0;
            }
          else if (lac_size)
            {
              yylac_stack_.clear ();
              yylen -= lac_size;
            }
          // Only afterwards look at the main stack.
          // We simulate popping elements by incrementing lac_top.
          lac_top += yylen;
        }
        // Keep top_state in sync with the updated stack.
        top_state = (yylac_stack_.empty ()
                     ? yystack_[lac_top].state
                     : yylac_stack_.back ());
        // Push the resulting state of the reduction.
        state_type state = yy_lr_goto_state_ (top_state, yyr1_[yyrule]);
        YYCDEBUG << " G" << int (state);
        yylac_stack_.push_back (state);
      }
  }

  // Establish the initial context if no initial context currently exists.
  bool
  parser::yy_lac_establish_ (symbol_kind_type yytoken)
  {
    /* Establish the initial context for the current lookahead if no initial
       context is currently established.

       We define a context as a snapshot of the parser stacks.  We define
       the initial context for a lookahead as the context in which the
       parser initially examines that lookahead in order to select a
       syntactic action.  Thus, if the lookahead eventually proves
       syntactically unacceptable (possibly in a later context reached via a
       series of reductions), the initial context can be used to determine
       the exact set of tokens that would be syntactically acceptable in the
       lookahead's place.  Moreover, it is the context after which any
       further semantic actions would be erroneous because they would be
       determined by a syntactically unacceptable token.

       yy_lac_establish_ should be invoked when a reduction is about to be
       performed in an inconsistent state (which, for the purposes of LAC,
       includes consistent states that don't know they're consistent because
       their default reductions have been disabled).

       For parse.lac=full, the implementation of yy_lac_establish_ is as
       follows.  If no initial context is currently established for the
       current lookahead, then check if that lookahead can eventually be
       shifted if syntactic actions continue from the current context.  */
    if (yy_lac_established_)
      return true;
    else
      {
#if YYDEBUG
        YYCDEBUG << "LAC: initial context established for "
                 << symbol_name (yytoken) << '\n';
#endif
        yy_lac_established_ = true;
        return yy_lac_check_ (yytoken);
      }
  }

  // Discard any previous initial lookahead context.
  void
  parser::yy_lac_discard_ (const char* event)
  {
   /* Discard any previous initial lookahead context because of Event,
      which may be a lookahead change or an invalidation of the currently
      established initial context for the current lookahead.

      The most common example of a lookahead change is a shift.  An example
      of both cases is syntax error recovery.  That is, a syntax error
      occurs when the lookahead is syntactically erroneous for the
      currently established initial context, so error recovery manipulates
      the parser stacks to try to find a new initial context in which the
      current lookahead is syntactically acceptable.  If it fails to find
      such a context, it discards the lookahead.  */
    if (yy_lac_established_)
      {
        YYCDEBUG << "LAC: initial context discarded due to "
                 << event << '\n';
        yy_lac_established_ = false;
      }
  }


  int
  parser::yy_syntax_error_arguments_ (const context& yyctx,
                                                 symbol_kind_type yyarg[], int yyargn) const
  {
    /* There are many possibilities here to consider:
       - If this state is a consistent state with a default action, then
         the only way this function was invoked is if the default action
         is an error action.  In that case, don't check for expected
         tokens because there are none.
       - The only way there can be no lookahead present (in yyla) is
         if this state is a consistent state with a default action.
         Thus, detecting the absence of a lookahead is sufficient to
         determine that there is no unexpected or expected token to
         report.  In that case, just report a simple "syntax error".
       - Don't assume there isn't a lookahead just because this state is
         a consistent state with a default action.  There might have
         been a previous inconsistent state, consistent state with a
         non-default action, or user semantic action that manipulated
         yyla.  (However, yyla is currently not documented for users.)
         In the first two cases, it might appear that the current syntax
         error should have been detected in the previous state when
         yy_lac_check was invoked.  However, at that time, there might
         have been a different syntax error that discarded a different
         initial context during error recovery, leaving behind the
         current lookahead.
    */

    if (!yyctx.lookahead ().empty ())
      {
        if (yyarg)
          yyarg[0] = yyctx.token ();
        int yyn = yyctx.expected_tokens (yyarg ? yyarg + 1 : yyarg, yyargn - 1);
        return yyn + 1;
      }
    return 0;
  }

  // Generate an error message.
  std::string
  parser::yysyntax_error_ (const context& yyctx) const
  {
    // Its maximum.
    enum { YYARGS_MAX = 5 };
    // Arguments of yyformat.
    symbol_kind_type yyarg[YYARGS_MAX];
    int yycount = yy_syntax_error_arguments_ (yyctx, yyarg, YYARGS_MAX);

    char const* yyformat = YY_NULLPTR;
    switch (yycount)
      {
#define YYCASE_(N, S)                         \
        case N:                               \
          yyformat = S;                       \
        break
      default: // Avoid compiler warnings.
        YYCASE_ (0, YY_("syntax error"));
        YYCASE_ (1, YY_("syntax error, unexpected %s"));
        YYCASE_ (2, YY_("syntax error, unexpected %s, expecting %s"));
        YYCASE_ (3, YY_("syntax error, unexpected %s, expecting %s or %s"));
        YYCASE_ (4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
        YYCASE_ (5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
      }

    std::string yyres;
    // Argument number.
    std::ptrdiff_t yyi = 0;
    for (char const* yyp = yyformat; *yyp; ++yyp)
      if (yyp[0] == '%' && yyp[1] == 's' && yyi < yycount)
        {
          yyres += symbol_name (yyarg[yyi++]);
          ++yyp;
        }
      else
        yyres += *yyp;
    return yyres;
  }


  const short parser::yypact_ninf_ = -705;

  const short parser::yytable_ninf_ = -315;

  const short
  parser::yypact_[] =
  {
     885,  -705,    33,   132,    54,  -705,  -705,  -705,    56,  -705,
    -705,  -705,  -705,  2381,    52,  -705,  2381,  -705,  2381,   146,
     178,  -705,  -705,  -705,  -705,  -705,    54,  2530,  -705,  -705,
    -705,  -705,  -705,  2381,  2381,  2381,  2381,  2381,    50,    99,
     104,   109,   114,   116,   127,   145,   155,   158,   161,  -705,
     167,   170,   180,   188,   192,   209,   214,   225,   228,   234,
     240,   241,   255,   269,   278,   279,   281,   282,   284,   290,
     293,   306,   307,   311,   313,   327,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,   908,  -705,   212,  -705,  -705,   129,
     322,   164,   372,  -705,  -705,  -705,  -705,  -705,  -705,  1097,
    1040,  1097,  2381,  1589,  -705,  1097,   -22,  -705,    17,  -705,
    -705,  -705,    17,  1532,  1589,    15,    29,    41,   226,  -705,
    1097,   193,   193,  2381,  -705,    69,  -705,  -705,   263,   263,
    -705,  -705,   263,   263,  2381,  1097,  -705,  2381,   263,   263,
     263,  2381,  1097,   263,  1097,  2381,   -76,    54,   315,  2381,
    2381,   642,  2381,  -705,  -705,  -705,  -705,   -74,   315,   315,
     315,   -54,   173,  -705,  -705,  -705,  -705,  -705,  2381,  2381,
    -705,   174,  -705,    47,  -705,  -705,  2381,   -54,   183,  2381,
     -76,  -705,   315,  -705,  -705,  -705,   291,   741,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,   388,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    1161,   384,   384,   384,   384,    59,  2381,  2381,  2381,  2381,
      32,  2381,  2381,   315,   315,  2381,   397,  2381,  2381,  2381,
    2381,  2381,   315,  2381,  2381,  2381,  2381,  2381,  2381,   964,
     964,   205,   315,   315,   315,   315,   315,   315,   315,   315,
     315,  2381,  2381,  2381,  2381,  2381,  2381,  2381,  2381,  2381,
    2381,  2381,  2381,  2381,  2381,  2381,  2381,  2381,  2381,  2381,
    2381,  2381,  2381,  -705,  -705,  -705,  -705,  -705,   352,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,   401,   412,  -705,   908,
    -705,  -705,   405,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,   418,   425,  -705,  -705,  -705,  -705,   427,  -705,
    1589,  -705,  -705,  -705,  -705,  -705,   908,   431,  -705,  -705,
    -705,   434,  -705,  -705,  -705,  -705,  -705,   441,   442,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,   432,  -705,   449,   403,
     464,  2022,   470,   472,   474,   477,   443,   493,  1344,   495,
     500,   502,   506,   508,   509,   510,   512,   513,   514,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,   524,
      69,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,   525,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
     526,   530,   537,   538,  2381,  -705,   541,   543,  -705,  -705,
     908,    75,   191,   544,  -705,   400,   410,   546,   552,   556,
    -705,  -705,   422,   444,   451,   819,  -705,  -705,   558,  -705,
    -705,   560,    54,  -705,  -705,   315,   562,   357,   564,  -705,
     159,  -705,   453,   564,   258,  -705,   357,   315,   363,  -705,
     315,  -705,  -705,  -705,  2381,  -705,   368,  -705,  2381,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  2381,
    2381,   315,  2381,  2381,  2381,  2381,  2381,   315,  2381,  2677,
    -705,   568,   579,   579,   579,   580,   575,   576,  1641,   579,
     577,   594,   579,    59,   597,   579,   598,   599,  1880,   595,
    2102,   602,  2126,   604,   579,   579,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,   600,   608,   609,   610,  -705,  -705,
      93,   607,  -705,   611,   618,   613,   615,   622,   623,   624,
     619,   579,  2290,   566,   566,   384,   384,   384,   384,   349,
     349,   349,   349,   349,   349,  2044,  1611,   601,   601,   601,
     354,   354,   354,   632,   636,    54,   631,  1777,  1834,  1097,
     331,  2381,   263,  -705,  2381,   589,   365,   417,  2381,  2381,
     634,   605,   637,   635,  1554,   639,   -22,   -22,   -22,   641,
     644,   643,   647,   649,   219,   650,   651,  2381,  2381,  2079,
    2267,   -22,   -22,    13,  1097,  1287,   -22,   263,   263,  2381,
    -705,  -705,  -705,  1392,  2381,  2381,  -705,  2381,  -705,  2381,
    2381,  2381,  -705,   258,  -705,  2381,  -705,   645,   452,  -705,
     357,  -705,  2381,  -705,  -705,  -705,   658,   357,  -705,   664,
    -705,   665,   669,  -705,   469,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  2381,
     666,   667,   668,  2381,  -705,  -705,  -705,   674,  -705,  2381,
     680,   681,  2381,   682,  2381,  2381,  -705,  -705,  -705,  2381,
    -705,  2381,   683,   684,  -705,  -705,  -705,  -705,   315,  -705,
     315,   675,  -705,   315,  -705,  -705,   315,   315,  2381,  -705,
     686,  -705,  2381,  -705,  -705,  -705,   687,   690,   691,   693,
     694,   695,   697,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,   704,   654,  2324,  -705,  2022,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,   707,   657,  -705,  -705,   705,   706,  -705,
    -705,  -705,  -705,    87,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,   315,   711,  -705,  -705,  -705,   709,   714,
    -705,  -705,  -705,   716,   559,  -705,  -705,  -705,   258,  -705,
     559,   559,   315,  -705,  -705,  -705,  -705,  -705,   579,  -705,
     713,  -705,  -705,   579,  -705,   579,   579,   579,   579,  -705,
    -705,  -705,   715,  2381,   717,   719,   720,   111,  -705,   728,
     727,    54,  -705,  -705,  -705,  -705,  -705,  -705,  -705,   -22,
     725,   731,   -22,   732,  -705,  -705,  2381,  2381,  -705,  -705,
    2381,  -705,   737,  2381,  -705,  2381,   729,  -705,   729,   729,
    -705,   733,  -705,   734,   736,   738,   742,   743,  -705,  -705,
     135,  -705,  -705,   315,  2381,  -705,  -705,  2381,  -705,  -705,
    -705,  -705,  -705,  -705,   908,   908,  -705,  2381,  -705,  -705,
    -705,  2381,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,   744,   754,    54,   747,   714,   739,   762,   762,   762,
    -705,  -705,  -705,  2381,  -705,  -705,   -79,  -705,   735,   760,
    2381,  2381,   761,   763,  -705,  -705
  };

  const short
  parser::yydefact_[] =
  {
       0,     5,     0,     7,     0,     1,     8,     9,    39,    12,
      13,     6,    40,     0,     0,   139,     0,   147,     0,     0,
      41,     4,    11,    19,    20,    21,     0,    25,    15,    16,
      17,    14,    18,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   268,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   232,   297,    36,    34,
      35,    38,    37,   231,   293,   229,     0,   230,    24,     0,
       0,     0,     0,   292,    43,    45,    42,    44,    10,     0,
       0,     0,     0,     0,   392,     0,   395,   397,     0,   400,
     401,   402,     0,     0,     0,     0,     0,     0,     0,   449,
       0,     0,     0,     0,   457,   455,   459,   461,     0,     0,
     463,   465,     0,     0,     0,     0,   469,     0,     0,     0,
       0,   474,     0,     0,     0,     0,   125,     0,     0,   164,
     168,     0,   166,    29,   137,   122,   157,     0,     0,     0,
       0,   317,     0,   156,   110,   202,   115,   120,   204,     0,
     181,   200,   116,   317,    32,   152,     0,   317,     0,   134,
     125,   155,     0,    30,    46,    50,     0,     0,    27,    28,
      77,    88,    87,    85,    86,    81,    82,    84,    74,    75,
      76,    73,    72,    61,    60,    62,    63,    64,    58,    55,
      56,    57,    89,    90,    91,    92,    93,    94,    78,    79,
      80,    53,    54,    65,    66,    67,    68,    69,    71,    70,
      51,    52,    59,    83,    26,   333,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,   345,   346,   347,   348,
     349,   350,   354,   352,   353,   351,   426,   427,   422,   423,
     424,   428,   429,   425,   355,   356,   357,   358,   359,   360,
     361,   362,   363,   364,   365,   366,   367,   368,   369,   370,
     371,   372,   373,   374,   375,   376,   377,   378,   379,   380,
       0,   254,   255,   233,   256,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   310,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    23,   140,    22,   148,   142,     0,   486,
     488,   490,   492,   494,   496,   498,     0,     0,   484,   223,
     382,   381,   515,   508,   509,   510,   511,   512,   513,   482,
     506,   507,     0,     0,   384,   383,   388,   387,     0,   210,
       0,   505,   503,   502,   504,   390,   227,     0,   500,   394,
     393,     0,   396,   520,   521,   522,   523,     0,     0,   515,
     398,   399,   403,   404,   407,   405,     0,   408,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   526,
     527,   451,   450,   519,   516,   517,   518,   452,   453,     0,
       0,   456,   460,   458,   464,   462,   466,   468,   467,     0,
     471,   472,   473,   475,   477,   476,   478,   480,   479,   107,
     108,     0,     0,     0,     0,   146,     0,    95,   211,   213,
       0,   229,   230,    95,   219,   229,   230,     0,   158,    95,
     215,   217,   229,   230,     0,     0,   188,   189,    95,   186,
     123,   191,     0,   319,   318,     0,   197,     0,    95,   206,
     229,   208,   230,    95,     0,   201,   118,     0,     0,   151,
       0,   199,   133,   135,     0,   124,     0,    48,     0,    97,
      98,    99,   100,   101,   104,   103,   102,   105,   106,   153,
       0,     0,   153,   153,     0,     0,     0,     0,     0,     0,
     291,     0,   295,   295,   295,     0,     0,     0,     0,   295,
       0,     0,   295,     0,     0,   295,     0,     0,     0,     0,
       0,     0,     0,     0,   295,   295,   324,   327,   323,   322,
     326,   321,   320,   325,     0,     0,     0,     0,   301,   308,
       0,     0,   312,     0,     0,     0,     0,     0,     0,     0,
       0,   295,     0,   242,   243,   250,   251,   252,   253,   236,
     241,   238,   237,   240,   239,   235,   234,   246,   245,   244,
     247,   248,   249,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   501,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   227,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     127,   128,   126,   129,     0,    96,   165,    96,   169,     0,
       0,    96,   167,    96,   185,     0,   190,     0,     0,   114,
     111,   112,    96,   205,   203,   183,    95,   119,   117,     0,
      33,     0,   131,    31,    47,   172,   178,   154,   170,   179,
     176,   177,   174,   173,   171,   180,   175,   334,   307,     0,
       0,     0,     0,     0,   262,   263,   259,     0,   289,     0,
       0,     0,     0,     0,     0,     0,   257,   290,   261,     0,
     258,     0,     0,     0,   264,   266,   265,   267,     0,   302,
       0,   311,   306,     0,   288,   304,     0,     0,     0,   303,
       0,   260,     0,   150,   138,   514,     0,     0,     0,     0,
       0,     0,     0,   485,   483,   385,   386,   389,   391,   406,
     409,     0,     0,     0,   481,     0,   420,   418,   419,   415,
     416,   417,   414,     0,     0,   524,   525,     0,     0,   528,
     530,   447,   448,     0,   431,   432,   433,   437,   436,   434,
     435,   439,   440,   441,   444,   443,   446,   442,   445,   438,
     454,   470,   109,     0,    95,   195,   212,   220,     0,    95,
     162,   216,   187,   192,     0,   198,   113,   207,    96,   182,
       0,     0,     0,    49,   296,   282,   281,   283,   295,   271,
       0,   279,   269,   295,   272,   295,   295,   295,   295,   270,
     278,   309,     0,     0,     0,     0,     0,     0,   280,     0,
     143,     0,   487,   489,   493,   497,   491,   495,   499,     0,
       0,     0,     0,     0,   529,   531,     0,     0,   430,   130,
      96,   194,    95,    96,   159,     0,   328,   184,   328,   328,
     132,     0,   300,     0,     0,     0,     0,     0,   285,   313,
     230,   286,   287,     0,     0,   299,   142,     0,   149,   412,
     413,   421,   410,   411,   225,   226,   196,    96,   160,   163,
     193,     0,   330,   330,   330,   284,   273,   275,   274,   277,
     276,     0,     0,     0,   144,    95,     0,   136,   316,   315,
     305,   298,   141,     0,   161,   329,     0,   145,     0,     0,
       0,     0,     0,     0,   331,   332
  };

  const short
  parser::yypgoto_[] =
  {
    -705,  -705,  -705,  -705,  -705,  -705,   -24,  -705,  -705,  -705,
    -705,  -705,  -705,   612,  -705,   614,  -705,  -142,  -264,  -705,
    -705,  -705,  -705,  -705,   616,  -395,  -171,  -705,  -653,  -705,
    -705,   271,  -601,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,   621,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -128,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -330,  -705,  -705,  -705,  -705,  -153,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,   106,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,   626,   101,   -89,  -705,   123,
    -705,   118,  -705,   125,  -127,  -705,  -705,   -68,  -705,     7,
    -130,  -705,   -18,   -12,   -23,  -393,   -67,  -705,  -705,  -705,
    -705,  -705,   -30,  -314,  -677,  -704,   292,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -110,  -109,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,  -705,
    -705,  -705,  -705,  -705,   -97,   289,   341,   -88,  -705,  -418,
    -705,  -705,  -705,  -705,    25,  -705,    71,   724,   726,   721,
    -107,   200,  -705,  -705
  };

  const short
  parser::yydefgoto_[] =
  {
       0,     2,     3,     4,     8,    21,    11,    22,    23,    24,
      25,    26,   185,   186,   526,   187,   518,    82,    83,    27,
     188,   527,   704,   189,   190,   676,   544,   191,   469,   192,
     507,   690,   691,   193,   194,   516,   698,   195,   196,   197,
     198,   199,   474,   200,   201,   522,   202,    28,    29,    89,
     623,   625,   869,    30,    31,    91,   624,   871,   203,   204,
     706,   205,   206,   207,   208,   829,   209,   210,   211,   212,
     213,   214,   215,   216,   217,   218,   219,   220,   514,   696,
     221,   498,   499,   222,    32,   223,   224,   824,   225,   226,
     227,   228,   229,   230,   231,   508,   509,   388,   477,   478,
     489,   490,   483,   484,   368,   479,   888,   643,   491,    84,
      85,   306,   470,    93,   511,   720,    87,   590,   593,   751,
     232,   233,   505,   585,   932,   947,   234,   235,   236,   237,
     238,   239,   240,   241,   242,   243,   244,   245,   246,   247,
     248,   249,   250,   251,   252,   253,   254,   422,   423,   255,
     256,   257,   258,   259,   260,   261,   262,   263,   264,   265,
     266,   267,   268,   269,   270,   271,   272,   273,   274,   275,
     276,   277,   278,   279,   280,   281,   282,   283,   284,   285,
     286,   287,   288,   289,   424,   370,   371,   409,   373,   374,
     375,   376,   377,   378,   633,   398,   379,   380,   381,   447,
     411,   438,   439,   440
  };

  const short
  parser::yytable_[] =
  {
      92,    86,    98,   645,    90,   413,   587,   456,   433,   434,
     645,   372,   372,   372,   463,   496,   548,   372,   402,   481,
     485,   435,   492,   419,   359,   358,   828,   420,   425,   427,
     958,   551,   372,     5,   449,   395,   556,   421,   510,   510,
     290,   291,   292,   293,   294,   415,   417,   372,   459,   426,
    -121,  -121,   471,   472,   372,    88,   372,     9,    10,   359,
     295,   359,    12,   359,   360,   361,   362,   363,   364,   365,
     809,   403,   404,   405,   406,   359,   174,   503,  -224,  -224,
    -224,   476,   482,   486,   473,   493,   959,   359,   678,   836,
     389,   500,   501,   502,   682,   450,   836,   183,   748,   886,
     887,   512,   512,   684,   749,   504,   369,   369,   369,   296,
     396,   389,   369,   693,   297,   525,   914,   391,   694,   298,
     396,   396,   915,   475,   299,   389,   300,   369,   397,   392,
     393,   394,    -2,   488,    78,    79,    80,   301,   416,   418,
    -314,   369,   369,   517,     6,     7,  -314,   520,   369,   369,
     451,   369,    94,    95,   401,   302,   480,   480,   519,   480,
     721,   722,  -294,  -294,  -294,   303,   727,   523,   304,   730,
    -294,   305,   733,    13,    14,   480,   480,   307,   503,   410,
     308,   742,   743,   412,    96,    97,    15,    38,    16,   401,
     309,   401,    17,   407,  -214,  -214,  -214,   408,   310,   452,
     453,    47,   311,   454,   455,   401,   504,    18,   760,   460,
     461,   462,   710,   711,   466,   353,   588,   401,    64,   312,
      66,   933,   934,    69,   313,    71,    72,    73,   765,   948,
     949,   797,   798,   557,   428,   314,   560,   561,   315,    77,
      78,    79,    80,    81,   316,   569,   443,   444,   445,   446,
     317,   318,   584,   586,   589,   591,   592,   594,   595,   596,
     597,   598,   599,   600,    19,   319,    20,    78,    79,    80,
      81,   358,   359,   360,   361,   362,   363,   364,   365,   320,
     429,   430,   431,   432,   552,   553,   554,   555,   321,   322,
     559,   323,   324,   562,   325,   564,   565,   566,   567,   731,
     326,   839,   571,   327,   573,   574,   575,   962,   558,   359,
     360,   361,   362,   363,   364,   365,   328,   329,   568,   601,
     570,   330,   572,   331,   548,   355,   528,   529,   530,   531,
     532,   533,   534,   535,   536,   537,   538,   332,   354,   602,
     603,   604,   605,   606,   607,   608,   609,   610,   611,   612,
     613,   614,   615,   616,   617,   618,   619,   620,   621,   622,
      38,   333,   334,   335,   336,   337,   338,   881,   335,   336,
     337,   338,   695,   356,    47,   357,   347,   348,   349,   350,
     351,   352,   506,   515,   539,   403,   404,   405,   406,   384,
     386,    64,   521,    66,   399,   549,    69,   293,    71,    72,
      73,   338,   407,  -221,  -221,  -221,   408,   563,   626,   441,
     629,   627,    77,  -222,  -222,  -222,   540,   541,   766,   767,
     768,   769,   628,   630,   457,  -228,  -228,  -228,   644,   891,
     631,   464,   632,   467,   894,   644,   634,   638,   687,   407,
     542,   385,   387,   408,   635,   543,   400,  -218,  -218,  -218,
     699,   636,   637,   701,   639,   901,  -209,  -209,  -209,   640,
     903,   442,   904,   905,   906,   907,    78,    79,    80,   641,
      38,   770,   771,   772,   709,   646,   458,   647,   686,   648,
     715,   673,   649,   465,    47,   468,   528,   529,   530,   531,
     532,   533,   534,   535,   536,   537,   538,   928,   651,   650,
     657,    64,   773,    66,   776,   658,    69,   659,    71,    72,
      73,   660,   702,   661,   662,   663,   705,   664,   665,   666,
     896,   707,    77,   775,   707,   707,   898,   899,   708,   667,
     668,   669,   712,   713,   714,   670,   716,   811,   813,   790,
     791,   496,   671,   672,   539,   481,   674,   485,   675,   677,
     954,   492,   792,   830,   679,   814,   815,   680,   787,   788,
     789,   681,   510,   683,   689,   685,   778,   688,   816,   692,
     779,   780,   700,   807,   808,   810,   540,   703,   819,   718,
     335,   336,   337,   338,   719,   723,   724,   725,   728,   801,
     802,   804,   806,   347,   348,   349,   350,   351,   352,   729,
     542,   764,   732,   734,   735,   543,   737,   739,   482,   741,
     486,   744,   750,   782,   493,   335,   336,   337,   338,   745,
     746,   747,   752,   753,   754,   512,   755,   756,   757,   758,
     759,   350,   351,   352,   568,   572,   369,   762,   369,   763,
     765,   396,   766,   781,   784,   396,   396,   783,   786,   421,
     834,   794,    33,   793,    34,    35,   795,   822,   796,   799,
     800,   835,   825,   838,   396,   396,   396,   396,    36,   840,
     841,   369,   369,   833,   842,    37,   843,   845,   846,   847,
     863,   861,   480,   862,   480,   849,   864,   369,   480,   865,
     866,   851,   852,   854,   859,   860,   897,   868,   872,   480,
     774,   873,   874,   777,   875,   876,   877,   844,   878,   879,
     880,   848,   882,   883,   884,   885,   890,   850,   892,   893,
     853,   895,   855,   856,   902,   913,   908,   857,   911,   858,
     912,   916,   917,   510,   920,   812,   817,   931,   820,   821,
     921,   923,   927,   960,   935,   936,   867,   937,   955,   938,
     870,   487,   953,   939,   940,   950,   889,   576,   577,   578,
     579,   580,   581,   582,   583,   951,   929,   956,   961,   494,
     964,   495,   965,   497,   945,   900,   545,   529,   530,   531,
     532,   533,   534,   535,   536,   537,   538,   697,   943,   832,
     572,   919,   603,   837,   922,   513,   910,    38,   826,   831,
     830,   524,   827,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
     909,   717,   436,   448,   437,     0,   941,   918,    76,    77,
      78,    79,    80,    81,   545,   529,   530,   531,   532,   533,
     534,   535,   536,   537,   538,   818,   546,   547,     0,     0,
     480,     0,     0,     0,     0,     0,     0,     0,   926,     0,
       0,     0,     0,   930,     0,    -3,     1,     0,    -3,    -3,
       0,    -3,     0,   924,   925,     0,   942,    -3,    -3,     0,
     369,     0,     0,     0,     0,   944,     0,     0,     0,     0,
       0,     0,     0,   946,     0,     0,     0,     0,     0,   952,
     333,   334,   335,   336,   337,   338,   339,   340,   341,   342,
     343,   344,   345,   346,   369,   347,   348,   349,   350,   351,
     352,   957,     0,   963,   546,     0,     0,     0,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,     0,     0,    -3,    -3,
       0,     0,    -3,    -3,     0,    -3,     0,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,     0,    -3,    -3,    -3,    -3,     0,
      -3,    -3,    -3,    -3,    -3,    -3,     0,     0,   358,     0,
      33,     0,    34,    35,     0,     0,    -3,     0,     0,     0,
       0,     0,     0,     0,     0,     0,    36,     0,     0,     0,
       0,     0,     0,    37,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   359,   360,   361,   362,
     363,   364,   365,    -3,    -3,    -3,   382,   383,     0,     0,
       0,     0,     0,     0,     0,   358,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,    38,
       0,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,     0,     0,    47,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   359,   360,   361,   362,   363,   364,   365,
      64,     0,    66,     0,     0,    69,     0,    71,    72,    73,
       0,     0,   576,   577,   578,   579,   580,   581,   582,   583,
       0,    77,   550,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,   346,     0,   347,   348,
     349,   350,   351,   352,     0,    38,     0,     0,     0,     0,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,   366,    55,    56,    57,
     367,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,     0,     0,
       0,     0,     0,     0,     0,     0,    76,    77,    78,    79,
      80,    81,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,   366,    55,    56,    57,   367,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,   428,     0,    33,     0,    34,
      35,     0,     0,    76,    77,    78,    79,    80,    81,     0,
       0,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   359,   360,   361,   362,   363,   364,   365,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    33,     0,    34,    35,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      36,     0,     0,     0,     0,     0,     0,    37,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   361,     0,     0,     0,     0,   823,   652,   653,
     654,     0,   655,   656,   333,   334,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,   345,   346,     0,   347,
     348,   349,   350,   351,   352,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,   366,    55,    56,    57,   367,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,     0,     0,     0,     0,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,    38,
       0,     0,     0,     0,     0,    39,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,   642,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    33,     0,    34,    35,     0,     0,     0,     0,
      76,    77,    78,    79,    80,    81,     0,     0,   390,     0,
       0,     0,     0,     0,     0,    37,   785,   334,   335,   336,
     337,   338,   339,   340,   341,   342,   343,   344,   345,   346,
     391,   347,   348,   349,   350,   351,   352,     0,   414,     0,
       0,     0,   392,   393,   394,     0,     0,     0,     0,    33,
       0,    34,    35,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   390,     0,     0,     0,     0,
       0,     0,    37,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,     0,   391,   347,   348,
     349,   350,   351,   352,     0,     0,     0,     0,     0,   392,
     393,   394,   726,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,   346,     0,   347,   348,
     349,   350,   351,   352,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
       0,     0,     0,     0,     0,     0,     0,     0,    76,    77,
      78,    79,    80,    81,    38,     0,     0,     0,     0,     0,
      39,    40,    41,    42,    43,    44,    45,    46,    47,    48,
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,    61,    62,    63,    64,    65,    66,    67,    68,
      69,    70,    71,    72,    73,    74,    75,    33,     0,    34,
      35,     0,     0,     0,     0,    76,    77,    78,    79,    80,
      81,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     766,   767,   768,   769,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    33,     0,    34,    35,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      36,     0,     0,     0,     0,     0,     0,    37,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   770,   771,
     772,   736,   333,   334,   335,   336,   337,   338,   339,   340,
     341,   342,   343,   344,   345,   346,     0,   347,   348,   349,
     350,   351,   352,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,     0,     0,     0,     0,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,    38,
       0,     0,     0,     0,     0,    39,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    33,     0,    34,    35,     0,     0,     0,     0,
      76,    77,    78,    79,    80,    81,     0,     0,    36,     0,
       0,     0,     0,     0,     0,    37,   333,   334,   335,   336,
     337,   338,   339,   340,   341,   342,   343,   344,     0,     0,
     361,   347,   348,   349,   350,   351,   352,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    33,
       0,    34,    35,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    36,     0,     0,     0,     0,
       0,     0,    37,   738,   333,   334,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,   345,   346,     0,   347,
     348,   349,   350,   351,   352,     0,   803,   740,   333,   334,
     335,   336,   337,   338,   339,   340,   341,   342,   343,   344,
     345,   346,     0,   347,   348,   349,   350,   351,   352,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,   642,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
       0,     0,     0,     0,     0,     0,     0,     0,    76,    77,
      78,    79,    80,    81,    38,     0,     0,     0,     0,     0,
      39,    40,    41,    42,    43,    44,    45,    46,    47,    48,
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,    61,    62,    63,    64,    65,    66,    67,    68,
      69,    70,    71,    72,    73,    74,    75,    33,     0,    34,
      35,     0,     0,     0,     0,    76,    77,    78,    79,    80,
      81,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,   761,   333,   334,   335,   336,   337,   338,   339,   340,
     341,   342,   343,   344,   345,   346,     0,   347,   348,   349,
     350,   351,   352,   805,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    33,     0,    34,    35,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      36,     0,     0,     0,     0,     0,     0,    37,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   770,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,    33,     0,    34,    35,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    36,     0,     0,
       0,     0,     0,     0,    37,     0,     0,     0,     0,     0,
       0,     0,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,     0,     0,     0,     0,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,    38,
       0,     0,     0,     0,     0,    39,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,     0,     0,     0,     0,     0,     0,     0,     0,
      76,    77,    78,    79,    80,    81,    38,     0,     0,     0,
       0,     0,    39,    40,    41,    42,    43,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,     0,
       0,     0,     0,     0,     0,     0,     0,    76,    77,    78,
      79,    80,    81,    99,   100,   101,   102,   103,   104,   105,
     106,   107,   108,   109,   110,   111,   112,   113,   114,   115,
     116,   117,   118,   119,   120,   121,   122,   123,   124,   125,
     126,   127,   128,   129,   130,   131,   132,   133,   134,   135,
     136,   137,   138,   139,   140,   141,   142,   143,   144,   145,
     146,   147,   148,   149,   150,   151,   152,     0,     0,   153,
     154,     0,     0,   155,   156,     0,     0,   157,   158,     0,
       0,     0,     0,   159,   160,   161,     0,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,     0,
     174,     0,   175,   176,     0,   177,   178,   179,   180,   181,
     182,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   183,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   184,
      99,   100,   101,   102,   103,   104,   105,   106,   107,   108,
     109,   110,   111,   112,   113,   114,   115,   116,   117,   118,
     119,   120,   121,   122,   123,   124,   125,   126,   127,   128,
     129,   130,   131,   132,   133,   134,   135,   136,   137,   138,
     139,   140,   141,   142,   143,   144
  };

  const short
  parser::yycheck_[] =
  {
      18,    13,    26,   421,    16,   112,   320,   134,   118,   118,
     428,    99,   100,   101,   141,   157,   187,   105,   106,   149,
     150,   118,   152,     8,    46,     8,   679,   115,   116,   117,
     109,   295,   120,     0,   123,   103,   300,     8,   168,   169,
      33,    34,    35,    36,    37,   113,   114,   135,   137,     8,
       3,     4,   128,   129,   142,     3,   144,     3,     4,    46,
      10,    46,     6,    46,    47,    48,    49,    50,    51,    52,
      57,    54,    55,    56,    57,    46,   150,   131,     3,     4,
       5,   148,   149,   150,   160,   152,   165,    46,   483,   690,
     102,   158,   159,   160,   489,    26,   697,   171,     5,    12,
      13,   168,   169,   498,    11,   159,    99,   100,   101,    10,
     103,   123,   105,   508,    10,   182,     5,    48,   513,    10,
     113,   114,    11,   147,    10,   137,    10,   120,   103,    60,
      61,    62,     0,   151,   208,   209,   210,    10,   113,   114,
       5,   134,   135,   173,    12,    13,    11,   177,   141,   142,
     125,   144,     6,     7,   176,    10,   149,   150,   176,   152,
     553,   554,     3,     4,     5,    10,   559,   179,    10,   562,
      11,    10,   565,   117,   118,   168,   169,    10,   131,   108,
      10,   574,   575,   112,     6,     7,   130,   155,   132,   176,
      10,   176,   136,   176,     3,     4,     5,   180,    10,   128,
     129,   169,    10,   132,   133,   176,   159,   151,   601,   138,
     139,   140,   542,   543,   143,     3,    11,   176,   186,    10,
     188,   898,   899,   191,    10,   193,   194,   195,     9,   933,
     934,    12,    13,   300,     8,    10,   303,   304,    10,   207,
     208,   209,   210,   211,    10,   312,    53,    54,    55,    56,
      10,    10,   319,   320,   321,   322,   323,   324,   325,   326,
     327,   328,   329,   330,   208,    10,   210,   208,   209,   210,
     211,     8,    46,    47,    48,    49,    50,    51,    52,    10,
      54,    55,    56,    57,   296,   297,   298,   299,    10,    10,
     302,    10,    10,   305,    10,   307,   308,   309,   310,   563,
      10,   696,   314,    10,   316,   317,   318,   960,   301,    46,
      47,    48,    49,    50,    51,    52,    10,    10,   311,   331,
     313,    10,   315,    10,   495,     3,    35,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,    10,   209,   332,
     333,   334,   335,   336,   337,   338,   339,   340,   341,   342,
     343,   344,   345,   346,   347,   348,   349,   350,   351,   352,
     155,    12,    13,    14,    15,    16,    17,   785,    14,    15,
      16,    17,   514,   209,   169,     3,    27,    28,    29,    30,
      31,    32,   209,   209,    93,    54,    55,    56,    57,   100,
     101,   186,   209,   188,   105,     7,   191,   390,   193,   194,
     195,    17,   176,     3,     4,     5,   180,    10,    56,   120,
       5,    10,   207,     3,     4,     5,   125,   126,    53,    54,
      55,    56,    10,     5,   135,     3,     4,     5,   421,   824,
       5,   142,     5,   144,   829,   428,     5,     5,   505,   176,
     149,   100,   101,   180,    10,   154,   105,     3,     4,     5,
     517,    10,    10,   520,     5,   848,     3,     4,     5,    56,
     853,   120,   855,   856,   857,   858,   208,   209,   210,     5,
     155,    54,    55,    56,   541,     5,   135,     5,   502,     5,
     547,   474,     5,   142,   169,   144,    35,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,   892,     5,    56,
       5,   186,   629,   188,   631,     5,   191,     5,   193,   194,
     195,     5,   524,     5,     5,     5,   528,     5,     5,     5,
     834,   539,   207,   630,   542,   543,   840,   841,   540,     5,
       5,     5,   544,   545,   546,     5,   548,   664,   665,   649,
     649,   683,     5,     5,    93,   675,     5,   677,     5,     5,
     945,   681,   649,   680,     8,   665,   665,     5,   646,   647,
     648,     5,   692,     5,   207,     5,   634,     5,   665,     5,
     638,   639,   209,   661,   662,   663,   125,   209,   666,    11,
      14,    15,    16,    17,     5,     5,    11,    11,    11,   657,
     658,   659,   660,    27,    28,    29,    30,    31,    32,     5,
     149,   625,     5,     5,     5,   154,    11,     5,   675,     5,
     677,    11,     5,     8,   681,    14,    15,    16,    17,    11,
      11,    11,    11,     5,    11,   692,    11,     5,     5,     5,
      11,    30,    31,    32,   627,   628,   629,     5,   631,     3,
       9,   634,    53,     9,     9,   638,   639,    10,     9,     8,
       5,     8,    10,     9,    12,    13,     9,   669,     9,     9,
       9,   209,   674,     5,   657,   658,   659,   660,    26,     5,
       5,   664,   665,   685,     5,    33,   207,    11,    11,    11,
       5,   748,   675,   750,   677,    11,   753,   680,   681,   756,
     757,    11,    11,    11,    11,    11,   838,    11,    11,   692,
     629,    11,    11,   632,    11,    11,    11,   719,    11,     5,
      56,   723,     5,    56,     9,     9,     5,   729,     9,     5,
     732,     5,   734,   735,    11,     5,    11,   739,    11,   741,
      11,     3,     5,   863,     9,   664,   665,     8,   667,   668,
       9,     9,     5,     8,    11,    11,   758,    11,     9,    11,
     762,   109,     5,    11,    11,    11,   823,   198,   199,   200,
     201,   202,   203,   204,   205,    11,   893,     5,     8,   157,
       9,   157,     9,   157,   927,   842,    35,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,   516,   916,   683,
     783,   879,   785,   692,   882,   169,   863,   155,   675,   681,
     927,   180,   677,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
     863,   549,   118,   122,   118,    -1,   913,   871,   206,   207,
     208,   209,   210,   211,    35,    36,    37,    38,    39,    40,
      41,    42,    43,    44,    45,   665,   125,   126,    -1,    -1,
     863,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   890,    -1,
      -1,    -1,    -1,   895,    -1,     0,     1,    -1,     3,     4,
      -1,     6,    -1,   886,   887,    -1,   914,    12,    13,    -1,
     893,    -1,    -1,    -1,    -1,   917,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   931,    -1,    -1,    -1,    -1,    -1,   943,
      12,    13,    14,    15,    16,    17,    18,    19,    20,    21,
      22,    23,    24,    25,   927,    27,    28,    29,    30,    31,
      32,   953,    -1,   961,   125,    -1,    -1,    -1,    63,    64,
      65,    66,    67,    68,    69,    70,    71,    72,    73,    74,
      75,    76,    77,    78,    79,    80,    81,    82,    83,    84,
      85,    86,    87,    88,    89,    90,    91,    92,    93,    94,
      95,    96,    97,    98,    99,   100,   101,   102,   103,   104,
     105,   106,   107,   108,   109,   110,   111,   112,   113,   114,
     115,   116,   117,   118,   119,   120,    -1,    -1,   123,   124,
      -1,    -1,   127,   128,    -1,   130,    -1,   132,   133,   134,
     135,   136,   137,   138,   139,   140,   141,   142,   143,   144,
     145,   146,   147,   148,    -1,   150,   151,   152,   153,    -1,
     155,   156,   157,   158,   159,   160,    -1,    -1,     8,    -1,
      10,    -1,    12,    13,    -1,    -1,   171,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    26,    -1,    -1,    -1,
      -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    46,    47,    48,    49,
      50,    51,    52,   208,   209,   210,    56,    57,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,     8,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   155,
      -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    -1,    -1,   169,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    46,    47,    48,    49,    50,    51,    52,
     186,    -1,   188,    -1,    -1,   191,    -1,   193,   194,   195,
      -1,    -1,   198,   199,   200,   201,   202,   203,   204,   205,
      -1,   207,    11,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    -1,    27,    28,
      29,    30,    31,    32,    -1,   155,    -1,    -1,    -1,    -1,
      -1,   161,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   206,   207,   208,   209,
     210,   211,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,     8,    -1,    10,    -1,    12,
      13,    -1,    -1,   206,   207,   208,   209,   210,   211,    -1,
      -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    46,    47,    48,    49,    50,    51,    52,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    10,    -1,    12,    13,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    48,    -1,    -1,    -1,    -1,     5,    54,    55,
      56,    -1,    58,    59,    12,    13,    14,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    -1,    27,
      28,    29,    30,    31,    32,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,   155,
      -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,   165,
     166,   167,   168,   169,   170,   171,   172,   173,   174,   175,
     176,   177,   178,   179,   180,   181,   182,   183,   184,   185,
     186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
     196,   197,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
     206,   207,   208,   209,   210,   211,    -1,    -1,    26,    -1,
      -1,    -1,    -1,    -1,    -1,    33,    12,    13,    14,    15,
      16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
      48,    27,    28,    29,    30,    31,    32,    -1,    56,    -1,
      -1,    -1,    60,    61,    62,    -1,    -1,    -1,    -1,    10,
      -1,    12,    13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,
      -1,    -1,    33,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    -1,    48,    27,    28,
      29,    30,    31,    32,    -1,    -1,    -1,    -1,    -1,    60,
      61,    62,    11,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    -1,    27,    28,
      29,    30,    31,    32,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   155,    -1,    -1,
      -1,    -1,    -1,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   206,   207,
     208,   209,   210,   211,   155,    -1,    -1,    -1,    -1,    -1,
     161,   162,   163,   164,   165,   166,   167,   168,   169,   170,
     171,   172,   173,   174,   175,   176,   177,   178,   179,   180,
     181,   182,   183,   184,   185,   186,   187,   188,   189,   190,
     191,   192,   193,   194,   195,   196,   197,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,   206,   207,   208,   209,   210,
     211,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      53,    54,    55,    56,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    10,    -1,    12,    13,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    54,    55,
      56,    11,    12,    13,    14,    15,    16,    17,    18,    19,
      20,    21,    22,    23,    24,    25,    -1,    27,    28,    29,
      30,    31,    32,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,   155,
      -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,   165,
     166,   167,   168,   169,   170,   171,   172,   173,   174,   175,
     176,   177,   178,   179,   180,   181,   182,   183,   184,   185,
     186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
     196,   197,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
     206,   207,   208,   209,   210,   211,    -1,    -1,    26,    -1,
      -1,    -1,    -1,    -1,    -1,    33,    12,    13,    14,    15,
      16,    17,    18,    19,    20,    21,    22,    23,    -1,    -1,
      48,    27,    28,    29,    30,    31,    32,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,
      -1,    12,    13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,
      -1,    -1,    33,    11,    12,    13,    14,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    -1,    27,
      28,    29,    30,    31,    32,    -1,    57,    11,    12,    13,
      14,    15,    16,    17,    18,    19,    20,    21,    22,    23,
      24,    25,    -1,    27,    28,    29,    30,    31,    32,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   155,    -1,    -1,
      -1,    -1,    -1,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   206,   207,
     208,   209,   210,   211,   155,    -1,    -1,    -1,    -1,    -1,
     161,   162,   163,   164,   165,   166,   167,   168,   169,   170,
     171,   172,   173,   174,   175,   176,   177,   178,   179,   180,
     181,   182,   183,   184,   185,   186,   187,   188,   189,   190,
     191,   192,   193,   194,   195,   196,   197,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,   206,   207,   208,   209,   210,
     211,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    11,    12,    13,    14,    15,    16,    17,    18,    19,
      20,    21,    22,    23,    24,    25,    -1,    27,    28,    29,
      30,    31,    32,    56,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    10,    -1,    12,    13,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    54,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    26,    -1,    -1,
      -1,    -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,   155,
      -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,   165,
     166,   167,   168,   169,   170,   171,   172,   173,   174,   175,
     176,   177,   178,   179,   180,   181,   182,   183,   184,   185,
     186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
     196,   197,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     206,   207,   208,   209,   210,   211,   155,    -1,    -1,    -1,
      -1,    -1,   161,   162,   163,   164,   165,   166,   167,   168,
     169,   170,   171,   172,   173,   174,   175,   176,   177,   178,
     179,   180,   181,   182,   183,   184,   185,   186,   187,   188,
     189,   190,   191,   192,   193,   194,   195,   196,   197,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   206,   207,   208,
     209,   210,   211,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    77,    78,    79,
      80,    81,    82,    83,    84,    85,    86,    87,    88,    89,
      90,    91,    92,    93,    94,    95,    96,    97,    98,    99,
     100,   101,   102,   103,   104,   105,   106,   107,   108,   109,
     110,   111,   112,   113,   114,   115,   116,    -1,    -1,   119,
     120,    -1,    -1,   123,   124,    -1,    -1,   127,   128,    -1,
      -1,    -1,    -1,   133,   134,   135,    -1,   137,   138,   139,
     140,   141,   142,   143,   144,   145,   146,   147,   148,    -1,
     150,    -1,   152,   153,    -1,   155,   156,   157,   158,   159,
     160,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   171,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   209,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    77,    78,    79,    80,    81,    82,
      83,    84,    85,    86,    87,    88,    89,    90,    91,    92,
      93,    94,    95,    96,    97,    98,    99,   100,   101,   102,
     103,   104,   105,   106,   107,   108
  };

  const short
  parser::yystos_[] =
  {
       0,     1,   213,   214,   215,     0,    12,    13,   216,     3,
       4,   218,     6,   117,   118,   130,   132,   136,   151,   208,
     210,   217,   219,   220,   221,   222,   223,   231,   259,   260,
     265,   266,   296,    10,    12,    13,    26,    33,   155,   161,
     162,   163,   164,   165,   166,   167,   168,   169,   170,   171,
     172,   173,   174,   175,   176,   177,   178,   179,   180,   181,
     182,   183,   184,   185,   186,   187,   188,   189,   190,   191,
     192,   193,   194,   195,   196,   197,   206,   207,   208,   209,
     210,   211,   229,   230,   321,   322,   325,   328,     3,   261,
     325,   267,   324,   325,     6,     7,     6,     7,   218,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,    90,    91,    92,    93,
      94,    95,    96,    97,    98,    99,   100,   101,   102,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,   116,   119,   120,   123,   124,   127,   128,   133,
     134,   135,   137,   138,   139,   140,   141,   142,   143,   144,
     145,   146,   147,   148,   150,   152,   153,   155,   156,   157,
     158,   159,   160,   171,   209,   224,   225,   227,   232,   235,
     236,   239,   241,   245,   246,   249,   250,   251,   252,   253,
     255,   256,   258,   270,   271,   273,   274,   275,   276,   278,
     279,   280,   281,   282,   283,   284,   285,   286,   287,   288,
     289,   292,   295,   297,   298,   300,   301,   302,   303,   304,
     305,   306,   332,   333,   338,   339,   340,   341,   342,   343,
     344,   345,   346,   347,   348,   349,   350,   351,   352,   353,
     354,   355,   356,   357,   358,   361,   362,   363,   364,   365,
     366,   367,   368,   369,   370,   371,   372,   373,   374,   375,
     376,   377,   378,   379,   380,   381,   382,   383,   384,   385,
     386,   387,   388,   389,   390,   391,   392,   393,   394,   395,
     321,   321,   321,   321,   321,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,   323,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    27,    28,    29,
      30,    31,    32,     3,   209,     3,   209,     3,     8,    46,
      47,    48,    49,    50,    51,    52,   176,   180,   316,   321,
     397,   398,   399,   400,   401,   402,   403,   404,   405,   408,
     409,   410,    56,    57,   397,   398,   397,   398,   309,   325,
      26,    48,    60,    61,    62,   319,   321,   406,   407,   397,
     398,   176,   399,    54,    55,    56,    57,   176,   180,   399,
     408,   412,   408,   412,    56,   319,   406,   319,   406,     8,
     399,     8,   359,   360,   396,   399,     8,   399,     8,    54,
      55,    56,    57,   359,   360,   396,   409,   410,   413,   414,
     415,   397,   398,    53,    54,    55,    56,   411,   411,   309,
      26,   406,   408,   408,   408,   408,   316,   397,   398,   309,
     408,   408,   408,   316,   397,   398,   408,   397,   398,   240,
     324,   128,   129,   160,   254,   218,   328,   310,   311,   317,
     321,   322,   328,   314,   315,   322,   328,   109,   324,   312,
     313,   320,   322,   328,   225,   227,   229,   236,   293,   294,
     328,   328,   328,   131,   159,   334,   209,   242,   307,   308,
     322,   326,   328,   307,   290,   209,   247,   334,   228,   324,
     334,   209,   257,   325,   254,   328,   226,   233,    35,    36,
      37,    38,    39,    40,    41,    42,    43,    44,    45,    93,
     125,   126,   149,   154,   238,    35,   125,   126,   238,     7,
      11,   230,   325,   325,   325,   325,   230,   328,   321,   325,
     328,   328,   325,    10,   325,   325,   325,   325,   321,   328,
     321,   325,   321,   325,   325,   325,   198,   199,   200,   201,
     202,   203,   204,   205,   328,   335,   328,   335,    11,   328,
     329,   328,   328,   330,   328,   328,   328,   328,   328,   328,
     328,   325,   321,   321,   321,   321,   321,   321,   321,   321,
     321,   321,   321,   321,   321,   321,   321,   321,   321,   321,
     321,   321,   321,   262,   268,   263,    56,    10,    10,     5,
       5,     5,     5,   406,     5,    10,    10,    10,     5,     5,
      56,     5,   180,   319,   321,   401,     5,     5,     5,     5,
      56,     5,    54,    55,    56,    58,    59,     5,     5,     5,
       5,     5,     5,     5,     5,     5,     5,     5,     5,     5,
       5,     5,     5,   321,     5,     5,   237,     5,   237,     8,
       5,     5,   237,     5,   237,     5,   218,   328,     5,   207,
     243,   244,     5,   237,   237,   229,   291,   243,   248,   328,
     209,   328,   325,   209,   234,   325,   272,   324,   325,   328,
     272,   272,   325,   325,   325,   328,   325,   338,    11,     5,
     327,   327,   327,     5,    11,    11,    11,   327,    11,     5,
     327,   230,     5,   327,     5,     5,    11,    11,    11,     5,
      11,     5,   327,   327,    11,    11,    11,    11,     5,    11,
       5,   331,    11,     5,    11,    11,     5,     5,     5,    11,
     327,    11,     5,     3,   218,     9,    53,    54,    55,    56,
      54,    55,    56,   316,   408,   412,   316,   408,   319,   319,
     319,     9,     8,    10,     9,    12,     9,   399,   399,   399,
     359,   360,   396,     9,     8,     9,     9,    12,    13,     9,
       9,   319,   319,    57,   319,    56,   319,   399,   399,    57,
     399,   316,   408,   316,   359,   360,   396,   408,   413,   399,
     408,   408,   325,     5,   299,   325,   311,   315,   240,   277,
     316,   313,   294,   325,     5,   209,   244,   308,     5,   237,
       5,     5,     5,   207,   325,    11,    11,    11,   325,    11,
     325,    11,    11,   325,    11,   325,   325,   325,   325,    11,
      11,   328,   328,     5,   328,   328,   328,   325,    11,   264,
     325,   269,    11,    11,    11,    11,    11,    11,    11,     5,
      56,   401,     5,    56,     9,     9,    12,    13,   318,   328,
       5,   237,     9,     5,   237,     5,   335,   229,   335,   335,
     328,   327,    11,   327,   327,   327,   327,   327,    11,   326,
     328,    11,    11,     5,     5,    11,     3,     5,   218,   399,
       9,     9,   399,     9,   321,   321,   325,     5,   237,   316,
     325,     8,   336,   336,   336,    11,    11,    11,    11,    11,
      11,   328,   324,   263,   325,   277,   324,   337,   337,   337,
      11,    11,   218,     5,   237,     9,     5,   325,   109,   165,
       8,     8,   240,   324,     9,     9
  };

  const short
  parser::yyr1_[] =
  {
       0,   212,   213,   214,   214,   215,   214,   216,   216,   216,
     217,   217,   218,   218,   219,   219,   219,   219,   219,   219,
     219,   219,   220,   221,   222,   223,   223,   223,   223,   224,
     226,   225,   228,   227,   229,   229,   229,   230,   230,   231,
     231,   231,   231,   231,   231,   231,   233,   232,   234,   234,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   236,
     236,   236,   236,   236,   236,   237,   237,   238,   238,   238,
     238,   238,   238,   238,   238,   238,   238,   239,   240,   240,
     242,   241,   243,   243,   244,   245,   247,   246,   248,   248,
     249,   250,   251,   252,   253,   254,   254,   254,   254,   255,
     255,   255,   255,   256,   257,   257,   258,   258,   259,   261,
     262,   260,   263,   264,   264,   264,   265,   267,   268,   266,
     269,   270,   271,   272,   272,   273,   274,   275,   276,   276,
     276,   276,   277,   277,   278,   278,   279,   279,   280,   280,
     281,   282,   283,   283,   283,   283,   284,   285,   286,   287,
     288,   290,   289,   291,   291,   292,   293,   293,   294,   295,
     296,   297,   297,   297,   298,   299,   299,   300,   300,   301,
     302,   303,   304,   305,   306,   306,   307,   307,   308,   308,
     309,   310,   310,   311,   311,   312,   312,   313,   313,   314,
     314,   315,   315,   316,   317,   318,   318,   319,   320,   321,
     321,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   323,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   324,   325,   326,   327,   327,   328,   328,   328,
     328,   328,   328,   328,   328,   328,   328,   328,   329,   329,
     330,   330,   331,   331,   331,   332,   333,   334,   334,   334,
     335,   335,   335,   335,   335,   335,   335,   335,   336,   336,
     337,   337,   337,   338,   338,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   340,   340,   341,   341,   341,   341,   342,   342,   343,
     344,   344,   345,   346,   346,   347,   347,   348,   349,   349,
     350,   351,   352,   353,   353,   354,   354,   354,   355,   355,
     356,   356,   357,   357,   358,   358,   358,   358,   358,   358,
     359,   360,   361,   361,   361,   361,   361,   361,   361,   361,
     362,   362,   363,   363,   364,   364,   365,   365,   366,   367,
     367,   368,   368,   368,   368,   368,   368,   369,   369,   370,
     371,   371,   372,   373,   374,   375,   375,   376,   377,   378,
     379,   380,   381,   382,   383,   384,   385,   386,   386,   387,
     388,   389,   390,   391,   392,   392,   393,   393,   394,   395,
     395,   396,   397,   397,   398,   398,   399,   399,   400,   400,
     401,   401,   402,   402,   403,   403,   404,   404,   405,   405,
     406,   406,   407,   407,   407,   407,   408,   408,   409,   409,
     409,   409,   409,   409,   409,   410,   411,   411,   411,   411,
     412,   412,   412,   412,   413,   413,   413,   413,   414,   414,
     415,   415
  };

  const signed char
  parser::yyr2_[] =
  {
       0,     2,     1,     0,     3,     0,     3,     0,     1,     1,
       2,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     3,     3,     2,     1,     2,     2,     2,     1,
       0,     3,     0,     3,     1,     1,     1,     1,     1,     0,
       1,     1,     2,     2,     2,     2,     0,     3,     0,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     0,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     2,     1,     3,
       0,     3,     1,     2,     1,     1,     0,     3,     0,     1,
       1,     1,     1,     2,     2,     0,     2,     2,     2,     3,
       5,     3,     5,     2,     0,     1,     7,     1,     5,     0,
       0,     9,     0,     1,     3,     5,     3,     0,     0,     7,
       0,     2,     1,     0,     1,     1,     1,     1,     2,     5,
       6,     8,     1,     3,     1,     3,     1,     3,     1,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     0,     4,     1,     3,     3,     1,     3,     1,     2,
       4,     2,     4,     6,     5,     1,     3,     2,     4,     2,
       1,     2,     1,     3,     1,     3,     1,     3,     1,     1,
       1,     1,     3,     1,     1,     1,     3,     1,     1,     1,
       3,     1,     1,     1,     1,     2,     2,     1,     1,     1,
       1,     1,     1,     2,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     2,     2,     2,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     0,     5,
       5,     5,     5,     7,     7,     7,     7,     7,     5,     5,
       5,     5,     5,     5,     7,     6,     6,     6,     4,     4,
       4,     3,     1,     1,     1,     0,     2,     1,     8,     6,
       6,     3,     4,     4,     4,     8,     4,     4,     1,     3,
       0,     2,     0,     3,     3,     7,     7,     0,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     0,     3,
       0,     6,     6,     1,     3,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     2,     2,     2,     2,     4,     4,     2,     2,     4,
       2,     4,     1,     2,     2,     1,     2,     1,     2,     2,
       1,     1,     1,     2,     2,     2,     4,     2,     2,     4,
       6,     6,     6,     6,     4,     4,     4,     4,     4,     4,
       3,     5,     1,     1,     1,     1,     1,     1,     1,     1,
       5,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     1,
       2,     2,     2,     2,     4,     1,     2,     1,     2,     1,
       2,     1,     2,     1,     2,     1,     2,     2,     2,     1,
       4,     2,     2,     2,     1,     2,     2,     2,     2,     2,
       2,     3,     1,     3,     1,     3,     1,     4,     1,     4,
       1,     4,     1,     4,     1,     4,     1,     4,     1,     4,
       1,     2,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     3,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     3,     3,     1,     1,     3,     4,
       3,     4
  };




#if YYDEBUG
  const short
  parser::yyrline_[] =
  {
       0,   400,   400,   403,   404,   406,   406,   416,   417,   422,
     432,   433,   436,   436,   443,   444,   445,   446,   447,   448,
     450,   451,   455,   467,   485,   504,   505,   506,   507,   511,
     517,   517,   526,   526,   537,   540,   543,   549,   552,   558,
     559,   562,   565,   568,   571,   575,   582,   582,   591,   594,
     601,   602,   603,   604,   605,   606,   607,   608,   609,   610,
     611,   612,   613,   614,   615,   616,   617,   618,   619,   620,
     621,   622,   623,   624,   625,   626,   627,   628,   629,   630,
     631,   632,   633,   634,   635,   636,   637,   638,   639,   643,
     644,   645,   646,   647,   648,   651,   651,   654,   657,   660,
     663,   666,   669,   672,   675,   678,   681,   687,   693,   702,
     721,   721,   728,   729,   733,   739,   745,   745,   753,   756,
     760,   766,   772,   778,   784,   790,   793,   796,   799,   805,
     812,   819,   824,   832,   842,   845,   849,   852,   858,   866,
     868,   866,   878,   884,   889,   894,   902,   910,   912,   910,
     922,   928,   934,   940,   943,   947,   953,   959,   965,   968,
     971,   977,   986,   989,   996,   999,  1003,  1006,  1010,  1013,
    1017,  1024,  1031,  1035,  1039,  1043,  1050,  1059,  1068,  1077,
    1084,  1091,  1091,  1102,  1105,  1111,  1114,  1115,  1119,  1125,
    1131,  1140,  1146,  1152,  1161,  1167,  1170,  1177,  1180,  1186,
    1192,  1198,  1205,  1210,  1213,  1217,  1224,  1225,  1229,  1232,
    1239,  1249,  1250,  1254,  1257,  1264,  1265,  1269,  1272,  1279,
    1280,  1284,  1287,  1294,  1301,  1308,  1312,  1319,  1326,  1333,
    1336,  1343,  1346,  1349,  1352,  1355,  1358,  1361,  1364,  1367,
    1370,  1373,  1376,  1379,  1382,  1385,  1388,  1391,  1394,  1397,
    1400,  1403,  1406,  1409,  1412,  1415,  1418,  1421,  1424,  1427,
    1430,  1433,  1436,  1440,  1443,  1446,  1449,  1452,  1455,  1455,
    1461,  1464,  1467,  1470,  1473,  1476,  1479,  1482,  1485,  1488,
    1491,  1494,  1497,  1500,  1503,  1506,  1509,  1514,  1519,  1522,
    1525,  1528,  1534,  1543,  1549,  1555,  1558,  1568,  1571,  1577,
    1583,  1589,  1592,  1595,  1599,  1603,  1606,  1609,  1631,  1634,
    1641,  1642,  1649,  1650,  1654,  1661,  1667,  1674,  1677,  1680,
    1686,  1689,  1692,  1695,  1698,  1701,  1704,  1707,  1713,  1716,
    1726,  1731,  1736,  1745,  1746,  1750,  1751,  1752,  1753,  1754,
    1755,  1756,  1757,  1758,  1759,  1760,  1761,  1762,  1763,  1764,
    1765,  1766,  1767,  1768,  1769,  1770,  1771,  1772,  1773,  1774,
    1775,  1776,  1777,  1778,  1779,  1780,  1781,  1782,  1783,  1784,
    1785,  1786,  1787,  1788,  1789,  1790,  1791,  1792,  1793,  1794,
    1795,  1799,  1803,  1809,  1813,  1816,  1819,  1826,  1830,  1836,
    1843,  1847,  1854,  1860,  1864,  1870,  1873,  1879,  1885,  1888,
    1894,  1900,  1906,  1912,  1915,  1921,  1925,  1929,  1935,  1939,
    1946,  1949,  1955,  1958,  1964,  1975,  1986,  1989,  1992,  1995,
    2000,  2003,  2012,  2013,  2014,  2015,  2016,  2017,  2018,  2019,
    2023,  2027,  2034,  2037,  2044,  2048,  2055,  2058,  2065,  2071,
    2075,  2085,  2089,  2092,  2095,  2099,  2102,  2109,  2113,  2122,
    2128,  2132,  2138,  2144,  2150,  2157,  2160,  2166,  2172,  2179,
    2185,  2192,  2198,  2205,  2211,  2218,  2224,  2235,  2239,  2245,
    2251,  2258,  2265,  2272,  2279,  2283,  2290,  2294,  2300,  2307,
    2311,  2319,  2325,  2326,  2332,  2335,  2343,  2344,  2348,  2349,
    2353,  2354,  2358,  2359,  2363,  2364,  2368,  2369,  2373,  2374,
    2378,  2379,  2385,  2388,  2391,  2394,  2399,  2399,  2402,  2405,
    2408,  2411,  2414,  2417,  2420,  2426,  2432,  2435,  2438,  2441,
    2447,  2450,  2453,  2456,  2462,  2465,  2468,  2471,  2477,  2478,
    2482,  2483
  };

  void
  parser::yy_stack_print_ () const
  {
    *yycdebug_ << "Stack now";
    for (stack_type::const_iterator
           i = yystack_.begin (),
           i_end = yystack_.end ();
         i != i_end; ++i)
      *yycdebug_ << ' ' << int (i->state);
    *yycdebug_ << '\n';
  }

  void
  parser::yy_reduce_print_ (int yyrule) const
  {
    int yylno = yyrline_[yyrule];
    int yynrhs = yyr2_[yyrule];
    // Print the symbols being reduced, and their result.
    *yycdebug_ << "Reducing stack by rule " << yyrule - 1
               << " (line " << yylno << "):\n";
    // The symbols being reduced.
    for (int yyi = 0; yyi < yynrhs; yyi++)
      YY_SYMBOL_PRINT ("   $" << yyi + 1 << " =",
                       yystack_[(yynrhs) - (yyi + 1)]);
  }
#endif // YYDEBUG


} // yy
#line 5993 "/build/rgbds/src/rgbds/src/asm/parser.cpp"

#line 2486 "asm/parser.y"


/******************** Semantic actions ********************/

void yy::parser::error(std::string const &str) {
	::error("%s\n", str.c_str());
}

static uint32_t strToNum(std::vector<int32_t> const &s) {
	uint32_t length = s.size();

	if (length == 1) {
		// The string is a single character with a single value,
		// which can be used directly as a number.
		return static_cast<uint32_t>(s[0]);
	}

	warning(WARNING_OBSOLETE, "Treating multi-unit strings as numbers is deprecated\n");

	for (int32_t v : s) {
		if (!checkNBit(v, 8, "All character units")) {
			break;
		}
	}

	uint32_t r = 0;

	for (uint32_t i = length < 4 ? 0 : length - 4; i < length; i++) {
		r <<= 8;
		r |= static_cast<uint8_t>(s[i]);
	}

	return r;
}

static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName) {
	error("%s: Invalid UTF-8 byte 0x%02hhX\n", functionName, byte);
}

static size_t strlenUTF8(std::string const &str, bool printErrors) {
	char const *ptr = str.c_str();
	size_t len = 0;
	uint32_t state = 0;

	for (uint32_t codepoint = 0; *ptr; ptr++) {
		uint8_t byte = *ptr;

		switch (decode(&state, &codepoint, byte)) {
		case 1:
			if (printErrors) {
				errorInvalidUTF8Byte(byte, "STRLEN");
			}
			state = 0;
			// fallthrough
		case 0:
			len++;
			break;
		}
	}

	// Check for partial code point.
	if (state != 0) {
		if (printErrors) {
			error("STRLEN: Incomplete UTF-8 character\n");
		}
		len++;
	}

	return len;
}

static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len) {
	char const *ptr = str.c_str();
	size_t index = 0;
	uint32_t state = 0;
	uint32_t codepoint = 0;
	uint32_t curPos = 1; // RGBASM strings are 1-indexed!

	// Advance to starting position in source string.
	while (ptr[index] && curPos < pos) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curPos++;
			break;
		}
		index++;
	}

	// A position 1 past the end of the string is allowed, but will trigger the
	// "Length too big" warning below if the length is nonzero.
	if (!ptr[index] && pos > curPos) {
		warning(
		    WARNING_BUILTIN_ARG, "STRSUB: Position %" PRIu32 " is past the end of the string\n", pos
		);
	}

	size_t startIndex = index;
	uint32_t curLen = 0;

	// Compute the result length in bytes.
	while (ptr[index] && curLen < len) {
		switch (decode(&state, &codepoint, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curLen++;
			break;
		}
		index++;
	}

	// Check for partial code point.
	if (state != 0) {
		error("STRSUB: Incomplete UTF-8 character\n");
		curLen++;
	}

	if (curLen < len) {
		warning(WARNING_BUILTIN_ARG, "STRSUB: Length too big: %" PRIu32 "\n", len);
	}

	return std::string(ptr + startIndex, ptr + index);
}

static size_t charlenUTF8(std::string const &str) {
	std::string_view view = str;
	size_t len;

	for (len = 0; charmap_ConvertNext(view, nullptr); len++) {}

	return len;
}

static std::string charsubUTF8(std::string const &str, uint32_t pos) {
	std::string_view view = str;
	size_t charLen = 1;

	// Advance to starting position in source string.
	for (uint32_t curPos = 1; charLen && curPos < pos; curPos++) {
		charLen = charmap_ConvertNext(view, nullptr);
	}

	std::string_view start = view;

	if (!charmap_ConvertNext(view, nullptr)) {
		warning(
		    WARNING_BUILTIN_ARG,
		    "CHARSUB: Position %" PRIu32 " is past the end of the string\n",
		    pos
		);
	}

	start = start.substr(0, start.length() - view.length());
	return std::string(start);
}

static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName) {
	// STRSUB and CHARSUB adjust negative `pos` arguments the same way,
	// such that position -1 is the last character of a string.
	if (pos < 0) {
		pos += len + 1;
	}
	if (pos < 1) {
		warning(WARNING_BUILTIN_ARG, "%s: Position starts at 1\n", functionName);
		pos = 1;
	}
	return static_cast<uint32_t>(pos);
}

static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep) {
	if (old.empty()) {
		warning(WARNING_EMPTY_STRRPL, "STRRPL: Cannot replace an empty string\n");
		return std::string(str);
	}

	std::string rpl;

	while (!str.empty()) {
		auto pos = str.find(old);
		if (pos == str.npos) {
			rpl.append(str);
			break;
		}
		rpl.append(str, 0, pos);
		rpl.append(rep);
		str.remove_prefix(pos + old.size());
	}

	return rpl;
}

static std::string
    strfmt(std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args) {
	std::string str;
	size_t argIndex = 0;

	for (size_t i = 0; spec[i] != '\0'; ++i) {
		int c = spec[i];

		if (c != '%') {
			str += c;
			continue;
		}

		c = spec[++i];

		if (c == '%') {
			str += c;
			continue;
		}

		FormatSpec fmt{};

		while (c != '\0') {
			fmt.useCharacter(c);
			if (fmt.isFinished()) {
				break;
			}
			c = spec[++i];
		}

		if (fmt.isEmpty()) {
			error("STRFMT: Illegal '%%' at end of format string\n");
			str += '%';
			break;
		}

		if (!fmt.isValid()) {
			error("STRFMT: Invalid format spec for argument %zu\n", argIndex + 1);
			str += '%';
		} else if (argIndex >= args.size()) {
			// Will warn after formatting is done.
			str += '%';
		} else if (args[argIndex].holds<uint32_t>()) {
			fmt.appendNumber(str, args[argIndex].get<uint32_t>());
		} else {
			fmt.appendString(str, args[argIndex].get<std::string>());
		}

		argIndex++;
	}

	if (argIndex < args.size()) {
		error("STRFMT: %zu unformatted argument(s)\n", args.size() - argIndex);
	} else if (argIndex > args.size()) {
		error(
		    "STRFMT: Not enough arguments for format spec, got: %zu, need: %zu\n",
		    args.size(),
		    argIndex
		);
	}

	return str;
}

static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue) {
	Expression oldExpr, constExpr, newExpr;
	int32_t newValue;

	oldExpr.makeSymbol(symName);
	constExpr.makeNumber(constValue);
	newExpr.makeBinaryOp(op, std::move(oldExpr), constExpr);
	newValue = newExpr.getConstVal();
	sym_AddVar(symName, newValue);
}

static void failAssert(AssertionType type) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed\n");
	case ASSERT_ERROR:
		error("Assertion failed\n");
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed\n");
		break;
	}
}

static void failAssertMsg(AssertionType type, std::string const &message) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed: %s\n", message.c_str());
	case ASSERT_ERROR:
		error("Assertion failed: %s\n", message.c_str());
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed: %s\n", message.c_str());
		break;
	}
}
